#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_layer3_virtual_networks_count_v1_info
short_description: Information module for Sda Layer3 Virtual Networks Count V1
description:
- Get all Sda Layer3 Virtual Networks Count V1.
- Returns the count of layer 3 virtual networks that match the provided query parameters.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  fabricId:
    description:
    - FabricId query parameter. ID of the fabric the layer 3 virtual network is assigned to.
    type: str
  anchoredSiteId:
    description:
    - AnchoredSiteId query parameter. Fabric ID of the fabric site the layer 3 virtual network is anchored at.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA GetLayer3VirtualNetworksCountV1
  description: Complete reference of the GetLayer3VirtualNetworksCountV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-layer-3-virtual-networks-count
notes:
  - SDK Method used are
    sda.Sda.get_layer3_virtual_networks_count_v1,

  - Paths used are
    get /dna/intent/api/v1/sda/layer3VirtualNetworks/count,

"""

EXAMPLES = r"""
- name: Get all Sda Layer3 Virtual Networks Count V1
  cisco.dnac.sda_layer3_virtual_networks_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    fabricId: string
    anchoredSiteId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: images_site_wise_product_names_count_v1_info
short_description: Information module for Images Site Wise Product Names Count V1
description:
- Get all Images Site Wise Product Names Count V1.
- >
   Returns count of assigned network device product for a given image identifier. Refer `/dna/intent/api/v1/images`
   API for obtaining `imageId`.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  imageId:
    description:
    - ImageId path parameter. Software image identifier. Refer `/dna/intent/api/v/images` API for obtaining `imageId`.
    type: str
  productName:
    description:
    - >
      ProductName query parameter. Filter with network device product name. Supports partial case-insensitive
      search. A minimum of 3 characters are required for search.
    type: str
  productId:
    description:
    - ProductId query parameter. Filter with product ID (PID).
    type: str
  recommended:
    description:
    - >
      Recommended query parameter. Filter with recommended source. If `CISCO` then the network device product
      assigned was recommended by Cisco and `USER` then the user has manually assigned. Available values CISCO,
      USER.
    type: str
  assigned:
    description:
    - >
      Assigned query parameter. Filter with the assigned/unassigned, `ASSIGNED` option will filter network device
      products that are associated with the given image. The `NOT_ASSIGNED` option will filter network device
      products that have not yet been associated with the given image but apply to it. Available values ASSIGNED,
      NOT_ASSIGNED.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) RetrievesTheCountOfAssignedNetworkDeviceProductsV1
  description: Complete reference of the RetrievesTheCountOfAssignedNetworkDeviceProductsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-count-of-assigned-network-device-products
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.retrieves_the_count_of_assigned_network_device_products_v1,

  - Paths used are
    get /dna/intent/api/v1/images/{imageId}/siteWiseProductNames/count,

"""

EXAMPLES = r"""
- name: Get all Images Site Wise Product Names Count V1
  cisco.dnac.images_site_wise_product_names_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    productName: string
    productId: string
    recommended: string
    assigned: string
    imageId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Dag Wieers (@dagwieers)
# Copyright: (c) 2023, Tim Cragg (@timcragg) <tcragg@cisco.com>
# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_tenant_action_rule_profile
short_description: Manage action rule profiles (rtctrl:AttrP)
description:
- Manage action rule profiles on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
    aliases: [ tenant_name ]
  action_rule:
    description:
    - The name of the action rule profile.
    type: str
    aliases: [action_rule_name, name ]
  set_community:
    description:
    - The set action rule based on communities.
    - To delete this attribute, pass an empty dictionary.
    type: dict
    suboptions:
      community:
        description:
        - The community value.
        type: str
      criteria:
        description:
        - The community criteria.
        - The option to append or replace the community value.
        type: str
        choices: [ append, replace, none ]
  set_dampening:
    description:
    - The set action rule based on dampening.
    - To delete this attribute, pass an empty dictionary.
    type: dict
    suboptions:
      half_life:
        description:
        - The half life value (minutes).
        type: int
      max_suppress_time:
        description:
        - The maximum suppress time value (minutes).
        type: int
      reuse:
        description:
        - The reuse limit value.
        type: int
      suppress:
        description:
        - The suppress limit value.
        type: int
  set_next_hop:
    description:
    - The set action rule based on the next hop address.
    - To delete this attribute, pass an empty string.
    type: str
  next_hop_propagation:
    description:
    - The set action rule based on nexthop unchanged configuration.
    - Can not be configured along with C(set_route_tag).
    - Can not be configured for APIC version 4.2 and prior.
    - The APIC defaults to C(false) when unset.
    type: bool
  multipath:
    description:
    - Set action rule based on set redistribute multipath configuration.
    - Can not be configured along with C(set_route_tag).
    - Can not be configured for APIC version 4.2 and prior.
    - The APIC defaults to C(false) when unset.
    type: bool
  set_preference:
    description:
    - The set action rule based on preference.
    - To delete this attribute, pass an empty string.
    type: str
  set_metric:
    description:
    - The set action rule based on metric.
    - To delete this attribute, pass an empty string.
    type: str
  set_metric_type:
    description:
    - The set action rule based on a metric type.
    - To delete this attribute, pass an empty string.
    type: str
    choices: [ ospf_type_1, ospf_type_2, "" ]
  set_route_tag:
    description:
    - The set action rule based on route tag.
    - Can not be configured along with C(next_hop_propagation) and C(multipath).
    - To delete this attribute, pass an empty string.
    type: str
  set_weight:
    description:
    - The set action rule based on weight.
    - To delete this attribute, pass an empty string.
    type: str
  description:
    description:
    - The description for the action rule profile.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(rtctrl:AttrP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
- Tim Cragg (@timcragg)
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Create a action rule profile
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    set_preference: 100
    set_weight: 100
    set_metric: 100
    set_metric_type: ospf_type_1
    set_next_hop: 1.1.1.1
    next_hop_propagation: true
    multipath: true
    set_community:
      community: no-advertise
      criteria: replace
    set_dampening:
      half_life: 10
      reuse: 1
      suppress: 10
      max_suppress_time: 100
    state: present
  delegate_to: localhost

- name: Delete action rule profile's children
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    set_preference: ""
    set_weight: ""
    set_metric: ""
    set_metric_type: ""
    set_next_hop: ""
    next_hop_propagation: false
    multipath: false
    set_community: {}
    set_dampening: {}
    state: present
  delegate_to: localhost

- name: Delete a action rule profile
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    state: absent
  delegate_to: localhost

- name: Query all action rule profiles
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific action rule profile
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    action_rule_set_comm_spec,
    action_rule_set_dampening_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import MATCH_ACTION_RULE_SET_METRIC_TYPE_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        action_rule=dict(type="str", aliases=["action_rule_name", "name"]),  # Not required for querying all objects
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        set_community=dict(type="dict", options=action_rule_set_comm_spec()),
        set_dampening=dict(type="dict", options=action_rule_set_dampening_spec()),
        set_next_hop=dict(type="str"),
        next_hop_propagation=dict(type="bool"),
        multipath=dict(type="bool"),
        set_preference=dict(type="str"),
        set_metric=dict(type="str"),
        set_metric_type=dict(type="str", choices=["ospf_type_1", "ospf_type_2", ""]),
        set_route_tag=dict(type="str"),
        set_weight=dict(type="str"),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["action_rule", "tenant"]],
            ["state", "present", ["action_rule", "tenant"]],
        ],
    )

    action_rule = module.params.get("action_rule")
    description = module.params.get("description")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)

    # This dict contains the name of the child classes as well as the corresping attribute input (and attribute name if the input is a string)
    # this dict is deviating from normal child classes list structure in order to determine which child classes should be created, modified, deleted or ignored.
    child_classes = dict(
        rtctrlSetComm=dict(attribute_input=module.params.get("set_community")),
        rtctrlSetDamp=dict(attribute_input=module.params.get("set_dampening")),
        rtctrlSetNh=dict(attribute_input=module.params.get("set_next_hop"), attribute_name="addr"),
        rtctrlSetPref=dict(attribute_input=module.params.get("set_preference"), attribute_name="localPref"),
        rtctrlSetRtMetric=dict(attribute_input=module.params.get("set_metric"), attribute_name="metric"),
        rtctrlSetRtMetricType=dict(
            attribute_input=MATCH_ACTION_RULE_SET_METRIC_TYPE_MAPPING.get(module.params.get("set_metric_type")), attribute_name="metricType"
        ),
        rtctrlSetTag=dict(attribute_input=module.params.get("set_route_tag"), attribute_name="tag"),
        rtctrlSetWeight=dict(attribute_input=module.params.get("set_weight"), attribute_name="weight"),
    )

    # This condition deal with child classes which do not exist in APIC version 4.2 and prior.
    additional_child_classes = dict(
        rtctrlSetNhUnchanged=dict(attribute_input=module.params.get("next_hop_propagation")),
        rtctrlSetRedistMultipath=dict(attribute_input=module.params.get("multipath")),
    )
    for class_name, attribute in additional_child_classes.items():
        if attribute.get("attribute_input") is not None:
            child_classes[class_name] = attribute

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="rtctrlAttrP",
            aci_rn="attr-{0}".format(action_rule),
            module_object=action_rule,
            target_filter={"name": action_rule},
        ),
        child_classes=list(child_classes.keys()),
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        for class_name, attribute in child_classes.items():
            attribute_input = attribute.get("attribute_input")
            # This condition enables to user to keep its previous configurations if they are not passing anything in the payload.
            if attribute_input is not None:
                # This condition checks if the attribute input is a dict and checks if all of its values are None (stored as a boolean in only_none).
                only_none = False
                if isinstance(attribute_input, dict):
                    only_none = all(value is None for value in attribute_input.values())
                # This condition checks if the child object needs to be deleted depending on the type of the corresponding attribute input (bool, str, dict).
                if (attribute_input == "" or attribute_input is False or only_none) and isinstance(aci.existing, list) and len(aci.existing) > 0:
                    for child in aci.existing[0].get("rtctrlAttrP", {}).get("children", {}):
                        if child.get(class_name):
                            child_configs.append(
                                {
                                    class_name: dict(
                                        attributes=dict(status="deleted"),
                                    ),
                                }
                            )
                # This condition checks if the child object needs to be modified or created depending on the type of the corresponding attribute input.
                elif attribute_input != "" or attribute_input is True or attribute_input != {}:
                    if class_name == "rtctrlSetComm" and isinstance(attribute_input, dict):
                        child_configs.append(
                            {
                                class_name: dict(
                                    attributes=dict(
                                        community=attribute_input.get("community"),
                                        setCriteria=attribute_input.get("criteria"),
                                    ),
                                )
                            }
                        )
                    elif class_name == "rtctrlSetDamp" and isinstance(attribute_input, dict):
                        child_configs.append(
                            {
                                class_name: dict(
                                    attributes=dict(
                                        halfLife=attribute_input.get("half_life"),
                                        maxSuppressTime=attribute_input.get("max_suppress_time"),
                                        reuse=attribute_input.get("reuse"),
                                        suppress=attribute_input.get("suppress"),
                                    ),
                                )
                            }
                        )
                    elif class_name in ["rtctrlSetNhUnchanged", "rtctrlSetRedistMultipath"]:
                        child_configs.append({class_name: dict(attributes=dict(descr=""))})
                    else:
                        child_configs.append({class_name: dict(attributes={attribute.get("attribute_name"): attribute_input})})

        aci.payload(
            aci_class="rtctrlAttrP",
            class_config=dict(
                name=action_rule,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="rtctrlAttrP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: integrate_ise_v1
short_description: Resource module for Integrate Ise V1
description:
- Manage operation update of the resource Integrate Ise V1.
- >
   API to accept Cisco ISE server certificate for Cisco ISE server integration. Use 'Cisco ISE Server Integration
   Status' Intent API to check the integration status. This API can be used to retry the failed integration.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Cisco ISE Server Identifier. Use 'Get Authentication
      and Policy Servers' intent API to find the identifier.
    type: str
  isCertAcceptedByUser:
    description: Value true for accept, false for deny. Remove this field and send empty
      request payload ( {} ) to retry the failed integration.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for System Settings AcceptCiscoISEServerCertificateForCiscoISEServerIntegrationV1
  description: Complete reference of the AcceptCiscoISEServerCertificateForCiscoISEServerIntegrationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!accept-cisco-ise-server-certificate-for-cisco-ise-server-integration
notes:
  - SDK Method used are
    system_settings.SystemSettings.accept_cisco_ise_server_certificate_for_cisco_ise_server_integration_v1,

  - Paths used are
    put /dna/intent/api/v1/integrate-ise/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.dnac.integrate_ise_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
    isCertAcceptedByUser: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "object": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: command_runner_run_command_v1
short_description: Resource module for Command Runner Run Command V1
description:
- Manage operation create of the resource Command Runner Run Command V1.
- Submit request for read-only CLIs.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  commands:
    description: Commands to be executed.
    elements: str
    type: list
  description:
    description: Describe the details about the command request.
    type: str
  deviceUuids:
    description: Device Id of the device.
    elements: str
    type: list
  name:
    description: Name of the the request like getshowrun , deviceinterfacestatusCli.
    type: str
  timeout:
    description: The timeout value in unit of second. If no timeout provided wait till
      300sec.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Command Runner RunReadOnlyCommandsOnDevicesToGetTheirRealTimeConfigurationV1
  description: Complete reference of the RunReadOnlyCommandsOnDevicesToGetTheirRealTimeConfigurationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!run-read-only-commands-on-devices-to-get-their-real-time-configuration
notes:
  - SDK Method used are
    command_runner.CommandRunner.run_read_only_commands_on_devices,

  - Paths used are
    post /dna/intent/api/v1/network-device-poller/cli/read-request,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.command_runner_run_command_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    commands:
    - string
    description: string
    deviceUuids:
    - string
    name: string
    timeout: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_l2_interfaces config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.l2_interfaces import (
    L2_interfacesTemplate,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.utils.utils import (
    generate_switchport_trunk,
    normalize_interface,
    vlan_list_to_range,
    vlan_range_to_list,
)


class L2_interfaces(ResourceModule):
    """
    The ios_l2_interfaces config class
    """

    def __init__(self, module):
        super(L2_interfaces, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="l2_interfaces",
            tmplt=L2_interfacesTemplate(),
        )
        self.parsers = [
            "access.vlan",
            "access.vlan_name",
            "voice.vlan",
            "voice.vlan_tag",
            "voice.vlan_name",
            "mode",
            "trunk.encapsulation",
            "trunk.native_vlan",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {entry["name"]: entry for entry in self.want}
        haved = {entry["name"]: entry for entry in self.have}

        for each in wantd, haved:
            self.process_list_attrs(each)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in iteritems(haved) if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in iteritems(haved):
                if k not in wantd:
                    self._compare(want={}, have=have)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the L2_interfaces network resource.
        """
        begin = len(self.commands)
        self.compare(parsers=self.parsers, want=want, have=have)
        self.compare_list(want, have)
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render(want or have, "name", False))

    def compare_list(self, want, have):
        for vlan in ["allowed_vlans", "pruning_vlans"]:
            cmd_always = list(
                set(want.get("trunk", {}).get(vlan, [])) - set(have.get("trunk", {}).get(vlan, [])),
            )  # find vlans to create wrt have
            if self.state != "merged":
                rem_vlan = []
                for vl_no in have.get("trunk", {}).get(vlan, []):
                    if vl_no not in cmd_always and vl_no not in want.get("trunk", {}).get(vlan, []):
                        rem_vlan.append(vl_no)
                if (
                    not want.get("trunk", {}).get(vlan, []) and rem_vlan
                ):  # remove vlan all as want blank
                    self.commands.append(
                        "no switchport trunk {0} vlan".format(vlan.split("_", maxsplit=1)[0]),
                    )
                elif rem_vlan:  # remove excess vlans for replaced overridden with vlan entries
                    self.commands.append(
                        "switchport trunk {0} vlan remove {1}".format(
                            vlan.split("_", maxsplit=1)[0],
                            vlan_list_to_range(sorted(rem_vlan)),
                        ),
                    )
            if self.state != "deleted" and cmd_always:  # add configuration needed
                self.commands.extend(
                    generate_switchport_trunk(
                        vlan.split("_", maxsplit=1)[0],
                        have.get("trunk", {}).get(vlan, []),
                        vlan_list_to_range(sorted(cmd_always)),
                    ),
                )

    def process_list_attrs(self, param):
        if param:
            for _k, val in iteritems(param):
                val["name"] = normalize_interface(val["name"])
                if val.get("trunk"):
                    for vlan in ["allowed_vlans", "pruning_vlans"]:
                        if val.get("trunk").get(vlan):
                            val["trunk"][vlan] = vlan_range_to_list(val.get("trunk").get(vlan))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for eos_lag_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: eos_lag_interfaces
short_description: LAG interfaces resource module
description: This module manages attributes of link aggregation groups on Arista EOS
  devices.
version_added: 1.0.0
author: Nathaniel Case (@Qalthos)
notes:
- Tested against Arista EOS 4.24.6F
- This module works with connection C(network_cli). See the L(EOS Platform Options,../network/user_guide/platform_eos.html).
options:
  config:
    description: A list of link aggregation group configurations.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name of the port-channel interface of the link aggregation group (LAG) e.g.,
          Port-Channel5.
        type: str
        required: true
      members:
        description:
        - Ethernet interfaces that are part of the group.
        type: list
        elements: dict
        suboptions:
          member:
            description:
            - Name of ethernet interface that is a member of the LAG.
            type: str
          mode:
            description:
            - LAG mode for this interface.
            type: str
            choices:
            - 'active'
            - 'on'
            - 'passive'
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the EOS device by
      executing the command B(show running-config | section interfaces).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
    - The state of the configuration after module completion.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - rendered
    - gathered
    - parsed
    default: merged

"""

EXAMPLES = """

# Using merged

# Before state:
# -------------
#
# veos#show running-config | section interface
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2

- name: Merge provided LAG attributes with existing device configuration
  arista.eos.eos_lag_interfaces:
    config:
      - name: Port-Channel5
        members:
          - member: Ethernet2
            mode: "on"
    state: merged

# After state:
# ------------
#
# veos#show running-config | section interface
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2
#   channel-group 5 mode on


# Using replaced

# Before state:
# -------------
#
# veos#show running-config | section interface
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2

- name: Replace all device configuration of specified LAGs with provided configuration
  arista.eos.eos_lag_interfaces:
    config:
      - name: Port-Channel5
        members:
          - member: Ethernet2
            mode: "on"
    state: replaced

# After state:
# ------------
#
# veos#show running-config | section interface
# interface Ethernet1
# interface Ethernet2
#   channel-group 5 mode on


# Using overridden

# Before state:
# -------------
#
# veos#show running-config | section interface
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2

- name: Override all device configuration of all LAG attributes with provided configuration
  arista.eos.eos_lag_interfaces:
    config:
      - name: Port-Channel10
        members:
          - member: Ethernet2
            mode: "on"
    state: overridden

# After state:
# ------------
#
# veos#show running-config | section interface
# interface Ethernet1
# interface Ethernet2
#   channel-group 10 mode on


# Using deleted

# Before state:
# -------------
#
# veos#show running-config | section interface
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2
#   channel-group 5 mode on

- name: Delete LAG attributes of the given interfaces.
  arista.eos.eos_lag_interfaces:
    config:
      - name: Port-Channel5
        members:
          - member: Ethernet1
    state: deleted

# After state:
# ------------
#
# veos#show running-config | section interface
# interface Ethernet1
# interface Ethernet2
#   channel-group 5 mode on

# Using parsed:

# parsed.cfg
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2
#   channel-group 5 mode on

- name: Use parsed to convert native configs to structured data
  arista.eos.lag_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Output:
#   parsed:
#     - name: Port-Channel5
#       members:
#         - member: Ethernet2
#           mode: "on"
#         - member: Ethernet1
#           mode: "on"

# using rendered:

- name: Use Rendered to convert the structured data to native config
  arista.eos.eos_lag_interfaces:
    config:
      - name: Port-Channel5
        members:
          - member: Ethernet2
            mode: "on"
          - member: Ethernet1
            mode: "on"
    state: rendered
# -----------
# Output
# -----------
#
# rendered:

# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2
#   channel-group 5 mode on


# Using gathered:

# native config:
# interface Ethernet1
#   channel-group 5 mode on
# interface Ethernet2
#   channel-group 5 mode on

- name: Gather lldp_global facts from the device
  arista.eos.lldp_global:
    state: gathered

# Output:
#   gathered:
#     - name: Port-Channel5
#       members:
#         - member: Ethernet2
#           mode: on
#         - member: Ethernet1
#           mode: on
"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['command 1', 'command 2', 'command 3']
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.lag_interfaces.lag_interfaces import (
    Lag_interfacesArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.lag_interfaces.lag_interfaces import (
    Lag_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]
    module = AnsibleModule(
        argument_spec=Lag_interfacesArgs.argument_spec,
        required_if=required_if,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    result = Lag_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: trustsec_vn
short_description: Resource module for Trustsec VN
description:
- Manage operations create, update and delete of the resource Trustsec VN.
- Create Virtual Network.
- Update Virtual Network.
version_added: '2.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  additionalAttributes:
    description: JSON String of additional attributes for the Virtual Network.
    type: str
  id:
    description: Identifier of the Virtual Network.
    type: str
  lastUpdate:
    description: Timestamp for the last update of the Virtual Network.
    type: str
  name:
    description: Name of the Virtual Network.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for virtualNetwork
  description: Complete reference of the virtualNetwork API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!trustsec-openapi
notes:
  - SDK Method used are
    virtual_network.VirtualNetwork.create_virtual_network,
    virtual_network.VirtualNetwork.delete_virtual_network_by_id,
    virtual_network.VirtualNetwork.update_virtual_network_by_id,

  - Paths used are
    post /api/v1/trustsec/virtualnetwork,
    delete /api/v1/trustsec/virtualnetwork/{id},
    put /api/v1/trustsec/virtualnetwork/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.trustsec_vn:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    additionalAttributes: string
    id: string
    lastUpdate: string
    name: string

- name: Update by id
  cisco.ise.trustsec_vn:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    additionalAttributes: string
    id: string
    lastUpdate: string
    name: string

- name: Delete by id
  cisco.ise.trustsec_vn:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "additionalAttributes": "string",
        "id": "string",
        "lastUpdate": "string",
        "name": "string"
      }
    ]

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "additionalAttributes": "string",
          "id": "string",
          "lastUpdate": "string",
          "name": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: reserve_ip_subpool
short_description: Resource module for Reserve Ip Subpool
description:
- This module represents an alias of the module reserve_ip_subpool_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Id of reserve ip subpool to be deleted.
    type: str
  ipv4DhcpServers:
    description: IPv4 input for dhcp server ip example "1.1.1.1".
    elements: str
    type: list
  ipv4DnsServers:
    description: IPv4 input for dns server ip example "4.4.4.4".
    elements: str
    type: list
  ipv4GateWay:
    description: Gateway ip address details, example 175.175.0.1.
    type: str
    version_added: 4.0.0
  ipv4GlobalPool:
    description: IP v4 Global pool address with cidr, example 175.175.0.0/16.
    type: str
  ipv4Prefix:
    description: IPv4 prefix value is true, the ip4 prefix length input field is enabled
      , if it is false ipv4 total Host input is enable.
    type: bool
  ipv4PrefixLength:
    description: The ipv4 prefix length is required when ipv4prefix value is true.
    type: int
  ipv4Subnet:
    description: IPv4 Subnet address, example 175.175.0.0. Either ipv4Subnet or ipv4TotalHost
      needs to be passed if creating IPv4 subpool.
    type: str
  ipv4TotalHost:
    description: IPv4 total host is required when ipv4prefix value is false.
    type: int
  ipv6AddressSpace:
    description: If the value is omitted or false only ipv4 input are required, otherwise
      both ipv6 and ipv4 are required.
    type: bool
  ipv6DhcpServers:
    description: IPv6 format dhcp server as input example "2001 db8 1234".
    elements: str
    type: list
  ipv6DnsServers:
    description: IPv6 format dns server input example "2001 db8 1234".
    elements: str
    type: list
  ipv6GateWay:
    description: Gateway ip address details, example 2001 db8 85a3 0 100 1.
    type: str
  ipv6GlobalPool:
    description: IPv6 Global pool address with cidr this is required when Ipv6AddressSpace
      value is true, example 2001 db8 85a3 /64.
    type: str
  ipv6Prefix:
    description: Ipv6 prefix value is true, the ip6 prefix length input field is enabled
      , if it is false ipv6 total Host input is enable.
    type: bool
  ipv6PrefixLength:
    description: IPv6 prefix length is required when the ipv6prefix value is true.
    type: int
  ipv6Subnet:
    description: IPv6 Subnet address, example 2001 db8 85a3 0 100. Either ipv6Subnet
      or ipv6TotalHost needs to be passed if creating IPv6 subpool.
    type: str
  ipv6TotalHost:
    description: IPv6 total host is required when ipv6prefix value is false.
    type: int
  name:
    description: Name of the reserve ip sub pool.
    type: str
  siteId:
    description: SiteId path parameter. Site id to reserve the ip sub pool.
    type: str
  slaacSupport:
    description: Slaac Support.
    type: bool
  type:
    description: Type of the reserve ip sub pool.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings ReserveIPSubpoolV1
  description: Complete reference of the ReserveIPSubpoolV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!reserve-ip-subpool
- name: Cisco DNA Center documentation for Network Settings ReleaseReserveIPSubpoolV1
  description: Complete reference of the ReleaseReserveIPSubpoolV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!release-reserve-ip-subpool
- name: Cisco DNA Center documentation for Network Settings UpdateReserveIPSubpoolV1
  description: Complete reference of the UpdateReserveIPSubpoolV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-reserve-ip-subpool
notes:
  - SDK Method used are
    network_settings.NetworkSettings.release_reserve_ip_subpool_v1,
    network_settings.NetworkSettings.reserve_ip_subpool_v1,
    network_settings.NetworkSettings.update_reserve_ip_subpool_v1,

  - Paths used are
    post /dna/intent/api/v1/reserve-ip-subpool/{siteId},
    delete /dna/intent/api/v1/reserve-ip-subpool/{id},
    put /dna/intent/api/v1/reserve-ip-subpool/{siteId},
  - It should be noted that this module is an alias of reserve_ip_subpool_v1

"""

EXAMPLES = r"""
- name: Delete by id
  cisco.dnac.reserve_ip_subpool:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

- name: Create
  cisco.dnac.reserve_ip_subpool:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    ipv4DhcpServers:
    - string
    ipv4DnsServers:
    - string
    ipv4GateWay: string
    ipv4GlobalPool: string
    ipv4Prefix: true
    ipv4PrefixLength: 0
    ipv4Subnet: string
    ipv4TotalHost: 0
    ipv6AddressSpace: true
    ipv6DhcpServers:
    - string
    ipv6DnsServers:
    - string
    ipv6GateWay: string
    ipv6GlobalPool: string
    ipv6Prefix: true
    ipv6PrefixLength: 0
    ipv6Subnet: string
    ipv6TotalHost: 0
    name: string
    siteId: string
    slaacSupport: true
    type: string

- name: Update by id
  cisco.dnac.reserve_ip_subpool:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    id: string
    ipv4DhcpServers:
    - string
    ipv4DnsServers:
    - string
    ipv4GateWay: string
    ipv6AddressSpace: true
    ipv6DhcpServers:
    - string
    ipv6DnsServers:
    - string
    ipv6GateWay: string
    ipv6GlobalPool: string
    ipv6Prefix: true
    ipv6PrefixLength: 0
    ipv6Subnet: string
    ipv6TotalHost: 0
    name: string
    siteId: string
    slaacSupport: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ipam_server_setting_v1
short_description: Resource module for Ipam Server Setting V1
description:
- Manage operations create, update and delete of the resource Ipam Server Setting V1.
- >
   Creates configuration details of the external IPAM server. You should only create one external IPAM server; delete
   any existing external server before creating a new one.
- Deletes configuration details of the external IPAM server.
- Updates configuration details of the external IPAM server.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  password:
    description: The password for the external IPAM server login username.
    type: str
  provider:
    description: Type of external IPAM. Can be either INFOBLOX, BLUECAT or GENERIC.
    type: str
  serverName:
    description: A descriptive name of this external server, used for identification
      purposes.
    type: str
  serverUrl:
    description: The URL of this external server.
    type: str
  syncView:
    description: Synchronize the IP pools from the local IPAM to this external server.
    type: bool
  userName:
    description: The external IPAM server login username.
    type: str
  view:
    description: The view under which pools are created in the external IPAM server.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for System Settings CreatesConfigurationDetailsOfTheExternalIPAMServerV1
  description: Complete reference of the CreatesConfigurationDetailsOfTheExternalIPAMServerV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!creates-configuration-details-of-the-external-ipam-server
- name: Cisco DNA Center documentation for System Settings DeletesConfigurationDetailsOfTheExternalIPAMServerV1
  description: Complete reference of the DeletesConfigurationDetailsOfTheExternalIPAMServerV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-configuration-details-of-the-external-ipam-server
- name: Cisco DNA Center documentation for System Settings UpdatesConfigurationDetailsOfTheExternalIPAMServerV1
  description: Complete reference of the UpdatesConfigurationDetailsOfTheExternalIPAMServerV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!updates-configuration-details-of-the-external-ipam-server
notes:
  - SDK Method used are
    system_settings.SystemSettings.creates_configuration_details_of_the_external_ip_a_m_server_v1,
    system_settings.SystemSettings.deletes_configuration_details_of_the_external_ip_a_m_server_v1,
    system_settings.SystemSettings.updates_configuration_details_of_the_external_ip_a_m_server_v1,

  - Paths used are
    post /dna/intent/api/v1/ipam/serverSetting,
    delete /dna/intent/api/v1/ipam/serverSetting,
    put /dna/intent/api/v1/ipam/serverSetting,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.ipam_server_setting_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    password: string
    provider: string
    serverName: string
    serverUrl: string
    syncView: true
    userName: string
    view: string

- name: Delete all
  cisco.dnac.ipam_server_setting_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent

- name: Update all
  cisco.dnac.ipam_server_setting_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    password: string
    serverName: string
    serverUrl: string
    syncView: true
    userName: string
    view: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_egress_igw_info
version_added: 9.0.0
short_description: Gather information about AWS VPC Egress Only Internet gateway
description:
    - Gather information about AWS VPC Egress Only Internet gateway.
author: "Aubin Bikouo (@abikouo)"
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeEgressOnlyInternetGateways.html) for possible filters.
    type: dict
    default: {}
  egress_only_internet_gateway_ids:
    description:
      - Get details of specific VPC Egress Internet Gateway ID.
    type: list
    elements: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# # Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all VPC Egress only Internet Gateways for an account or profile
  amazon.aws.ec2_vpc_igw_info:
    region: ap-southeast-2
    profile: production

- name: Gather information about a filtered list of VPC Egress only Internet Gateways
  amazon.aws.ec2_vpc_igw_info:
    region: ap-southeast-2
    profile: production
    filters:
      "tag:Name": "igw-123"

- name: Gather information about a specific VPC egress only internet gateway by EgressOnlyInternetGatewayId
  amazon.aws.ec2_vpc_igw_info:
    region: ap-southeast-2
    profile: production
    egress_only_internet_gateway_ids: igw-c1231234
"""

RETURN = r"""
egress_only_internet_gateways:
    description: The AWS Egress only internet gateways.
    returned: always
    type: list
    contains:
        attachments:
            description: Information about the attachment of the egress-only internet gateway.
            returned: always
            type: list
            contains:
                state:
                    description: The current state of the attachment.
                    returned: always
                    type: str
                    sample: "available"
                vpc_id:
                    description: The ID of the VPC.
                    returned: always
                    type: str
                    sample: "vpc-02123b67"
        egress_only_internet_gateway_id:
            description: The ID of the egress-only internet gateway.
            returned: always
            type: str
            sample: "eigw-0123456789abcdef"
        tags:
            description: Any tags assigned to the egress-only internet gateway.
            returned: always
            type: dict
"""

from typing import Any
from typing import Dict
from typing import List

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_egress_only_internet_gateways
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def format_egress_only_internet_gateway(internet_gateway: Dict[str, Any]) -> Dict[str, Any]:
    tags = boto3_tag_list_to_ansible_dict(internet_gateway.get("Tags", []))
    egress_only_igw = {
        "EgressOnlyInternetGatewayId": internet_gateway["EgressOnlyInternetGatewayId"],
        "Attachments": internet_gateway["Attachments"],
        "Tags": tags,
    }
    return camel_dict_to_snake_dict(egress_only_igw, ignore_list=["Tags"])


def list_egress_only_internet_gateways(connection, module: AnsibleAWSModule) -> List[Dict[str, Any]]:
    params = dict()

    params["Filters"] = ansible_dict_to_boto3_filter_list(module.params.get("filters"))

    if module.params.get("egress_only_internet_gateway_ids"):
        params["EgressOnlyInternetGatewayIds"] = module.params.get("egress_only_internet_gateway_ids")

    try:
        egress_only_igws = describe_egress_only_internet_gateways(connection, **params)
        return [format_egress_only_internet_gateway(igw) for igw in egress_only_igws]
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, "Unable to describe egress only internet gateways")


def main() -> None:
    argument_spec = dict(
        filters=dict(type="dict", default=dict()),
        egress_only_internet_gateway_ids=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)
    client = module.client("ec2")

    results = list_egress_only_internet_gateways(client, module)
    module.exit_json(egress_only_internet_gateways=results)


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    name: validate
    author: Ganesh Nalawade (@ganeshrn)
    version_added: "1.0.0"
    short_description: Validate data with provided criteria
    description:
        - Validate I(data) with provided I(criteria) based on the validation I(engine).
    options:
      data:
        type: raw
        description:
        - Data that will be validated against I(criteria).
        - This option represents the value that is passed to the filter plugin in pipe format.
          For example C(config_data|ansible.utils.validate()), in this case C(config_data)
          represents this option.
        - For the type of I(data) that represents this value refer to the documentation of individual validator plugins.
        required: True
      criteria:
        type: raw
        description:
        - The criteria used for validation of value that represents I(data) options.
        - This option represents the first argument passed in the filter plugin.
          For example C(config_data|ansible.utils.validate(config_criteria)), in
          this case the value of C(config_criteria) represents this option.
        - For the type of I(criteria) that represents this value refer to the  documentation of individual validator plugins.
        required: True
      engine:
        type: str
        description:
        - The name of the validator plugin to use.
        - This option can be passed in lookup plugin as a key, value pair.
          For example C(config_data|ansible.utils.validate(config_criteria, engine='ansible.utils.jsonschema')), in
          this case the value C(ansible.utils.jsonschema) represents the engine to be use for data validation.
          If the value is not provided the default value that is C(ansible.utils.jsonschema) will be used.
        - The value should be in fully qualified collection name format that is
          C(<org-name>.<collection-name>.<validator-plugin-name>).
        default: ansible.utils.jsonschema
    notes:
    - For the type of options I(data) and I(criteria) refer to the individual validate plugin
      documentation that is represented in the value of I(engine) option.
    - For additional plugin configuration options refer to the individual validate plugin
      documentation that is represented by the value of I(engine) option.
    - The plugin configuration option can be either passed as C(key=value) pairs within filter plugin
      or environment variables.
    - The precedence of the I(validate) plugin configurable option is the variable passed within filter plugin
      as C(key=value) pairs followed by the environment variables.
"""

EXAMPLES = r"""
- name: set facts for data and criteria
  ansible.builtin.set_fact:
    data: "{{ lookup('ansible.builtin.file', './validate/data/show_interfaces_iosxr.json')}}"
    criteria: "{{ lookup('ansible.builtin.file', './validate/criteria/jsonschema/show_interfaces_iosxr.json')}}"

- name: validate data in json format using jsonschema by passing plugin configuration variable as key/value pairs
  ansible.builtin.set_fact:
    data_validity: "{{ data|ansible.utils.validate(criteria, engine='ansible.utils.jsonschema', draft='draft7') }}"
"""

RETURN = """
  _raw:
    description:
      - If data is valid returns empty list
      - If data is invalid returns list of errors in data
"""

from ansible.errors import AnsibleError, AnsibleFilterError
from ansible.module_utils._text import to_text

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    check_argspec,
)
from ansible_collections.ansible.utils.plugins.module_utils.common.utils import to_list
from ansible_collections.ansible.utils.plugins.plugin_utils.base.validate import _load_validator


ARGSPEC_CONDITIONALS = {}


def validate(*args, **kwargs):
    if len(args) < 2:
        raise AnsibleFilterError(
            "Missing either 'data' or 'criteria' value in filter input,"
            " refer 'ansible.utils.validate' filter plugin documentation for details",
        )

    params = {"data": args[0], "criteria": args[1]}
    if kwargs.get("engine"):
        params.update({"engine": kwargs["engine"]})

    valid, argspec_result, updated_params = check_argspec(
        DOCUMENTATION,
        "validate filter",
        schema_conditionals=ARGSPEC_CONDITIONALS,
        **params,
    )
    if not valid:
        raise AnsibleFilterError(
            "{argspec_result} with errors: {argspec_errors}".format(
                argspec_result=argspec_result.get("msg"),
                argspec_errors=argspec_result.get("errors"),
            ),
        )

    validator_engine, validator_result = _load_validator(
        engine=updated_params["engine"],
        data=updated_params["data"],
        criteria=updated_params["criteria"],
        kwargs=kwargs,
    )
    if validator_result.get("failed"):
        raise AnsibleFilterError(
            "validate lookup plugin failed with errors: {msg}".format(
                msg=validator_result.get("msg"),
            ),
        )

    try:
        result = validator_engine.validate()
    except AnsibleError as exc:
        raise AnsibleFilterError(to_text(exc, errors="surrogate_then_replace"))
    except Exception as exc:
        raise AnsibleFilterError(
            "Unhandled exception from validator '{validator}'. Error: {err}".format(
                validator=updated_params["engine"],
                err=to_text(exc, errors="surrogate_then_replace"),
            ),
        )

    return to_list(result.get("errors", []))


class FilterModule(object):
    """index_of"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"validate": validate}



# -*- coding: utf-8 -*-

# Copyright: (c) 2016 Matt Davis, <mdavis@ansible.com>
# Copyright: (c) 2016 Chris Houseknecht, <house@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):

    # Azure doc fragment
    DOCUMENTATION = r'''
options:
    plugin:
        description: marks this as an instance of the 'azure_rm' plugin
        required: true
        choices: ['azure_rm', 'azure.azcollection.azure_rm']
    include_vm_resource_groups:
        description: A list of resource group names to search for virtual machines. '\*' will include all resource
            groups in the subscription. Can also be set comma separated resource group names via the
            C(ANSIBLE_AZURE_VM_RESOURCE_GROUPS) environment variable.
        default: ['*']
    include_hcivm_resource_groups:
        description: A list of resource group names to search for Azure StackHCI virtual machines. '\*' will
            include all resource groups in the subscription.
        default: []
    include_vmss_resource_groups:
        description: A list of resource group names to search for virtual machine scale sets (VMSSs). '\*' will
            include all resource groups in the subscription.
        default: []
    fail_on_template_errors:
        description: When false, template failures during group and filter processing are silently ignored (eg,
            if a filter or group expression refers to an undefined host variable)
        choices: [True, False]
        default: True
    keyed_groups:
        description: Creates groups based on the value of a host variable. Requires a list of dictionaries,
            defining C(key) (the source dictionary-typed variable), C(prefix) (the prefix to use for the new group
            name), and optionally C(separator) (which defaults to C(_))
    conditional_groups:
        description: A mapping of group names to Jinja2 expressions. When the mapped expression is true, the host
            is added to the named group.
    hostvar_expressions:
        description: A mapping of hostvar names to Jinja2 expressions. The value for each host is the result of the
            Jinja2 expression (which may refer to any of the host's existing variables at the time this inventory
            plugin runs).
    exclude_host_filters:
        description: Excludes hosts from the inventory with a list of Jinja2 conditional expressions. Each
            expression in the list is evaluated for each host; when the expression is true, the host is excluded
            from the inventory.
        default: []
    include_host_filters:
        description: Include hosts from the inventory with a list of Jinja2 conditional expressions. Each
            expression in the list is evaluated for each host; when the expression is true, the host is included
            in the inventory, all hosts are includes in the inventory by default.
        default: [true]
    batch_fetch:
        description: To improve performance, results are fetched using an unsupported batch API. Disabling
            C(batch_fetch) uses a much slower serial fetch, resulting in many more round-trips. Generally only
            useful for troubleshooting.
        default: true
    default_host_filters:
        description: A default set of filters that is applied in addition to the conditions in
            C(exclude_host_filters) to exclude powered-off and not-fully-provisioned hosts. Set this to a different
            value or empty list if you need to include hosts in these states.
        default: ['powerstate != "running"', 'provisioning_state != "succeeded"']
    use_contrib_script_compatible_sanitization:
        description:
        - By default this plugin is using a general group name sanitization to create safe and usable group names for use in Ansible.
            This option allows you to override that, in efforts to allow migration from the old inventory script and
            matches the sanitization of groups when the script's C(replace_dash_in_groups) option is set to C(false).
            To replicate behavior of C(replace_dash_in_groups = true) with constructed groups,
            you will need to replace hyphens with underscores via the regex_replace filter for those entries.
        - For this to work you should also turn off the TRANSFORM_INVALID_GROUP_CHARS setting,
            otherwise the core engine will just use the standard sanitization on top.
        - This is not the default as such names break certain functionality as not all characters are valid Python identifiers
            which group names end up being used as.
        type: bool
        default: False
        version_added: '0.0.1'
    plain_host_names:
        description:
        - By default this plugin will use globally unique host names.
            This option allows you to override that, and use the name that matches the old inventory script naming.
        - This is not the default, as these names are not truly unique, and can conflict with other hosts.
            The default behavior will add extra hashing to the end of the hostname to prevent such conflicts.
        type: bool
        default: False
        version_added: '0.0.1'
    hostnames:
        description:
        - A list of Jinja2 expressions in order of precedence to compose inventory_hostname.
        - Ignores expression if result is an empty string or None value.
        - By default, inventory_hostname is generated to be globally unique based on the VM host name.
            See C(plain_host_names) for more details on the default.
        - An expression of C(default) will force using the default hostname generator if no previous hostname expression
            resulted in a valid hostname.
        - Use C(default_inventory_hostname) to access the default hostname generator's value in any of the Jinja2 expressions.
        type: list
        elements: str
        default: [default]
'''



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import Credential, CredentialType, Organization


@pytest.fixture
def cred_type():
    # Make a credential type which will be used by the credential
    ct = CredentialType.objects.create(
        name='Ansible Galaxy Token',
        inputs={"fields": [{"id": "token", "type": "string", "secret": True, "label": "Ansible Galaxy Secret Token Value"}], "required": ["token"]},
        injectors={
            "extra_vars": {
                "galaxy_token": "{{token}}",
            }
        },
    )
    return ct


@pytest.mark.django_db
def test_create_machine_credential(run_module, admin_user, organization):
    Organization.objects.create(name='test-org')
    # create the ssh credential type
    ct = CredentialType.defaults['ssh']()
    ct.save()
    # Example from docs
    result = run_module(
        'credential',
        dict(name='Test Machine Credential',
             organization=organization.name,
             credential_type='Machine',
             state='present'),
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result

    cred = Credential.objects.get(name='Test Machine Credential')
    assert cred.credential_type == ct

    assert result['name'] == "Test Machine Credential"
    assert result['id'] == cred.pk


@pytest.mark.django_db
def test_create_vault_credential(run_module, admin_user, organization):
    # https://github.com/ansible/ansible/issues/61324
    Organization.objects.create(name='test-org')
    ct = CredentialType.defaults['vault']()
    ct.save()

    result = run_module(
        'credential',
        dict(name='Test Vault Credential',
             organization=organization.name,
             credential_type='Vault',
             inputs={'vault_id': 'bar', 'vault_password': 'foobar'},
             state='present'),
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result

    cred = Credential.objects.get(name='Test Vault Credential')
    assert cred.credential_type == ct
    assert 'vault_id' in cred.inputs
    assert 'vault_password' in cred.inputs

    assert result['name'] == "Test Vault Credential"
    assert result['id'] == cred.pk


@pytest.mark.django_db
def test_missing_credential_type(run_module, admin_user, organization):
    Organization.objects.create(name='test-org')
    result = run_module('credential', dict(name='A credential', organization=organization.name, credential_type='foobar', state='present'), admin_user)
    assert result.get('failed', False), result
    assert 'credential_type' in result['msg']
    assert 'foobar' in result['msg']
    assert 'returned 0 items, expected 1' in result['msg']


@pytest.mark.django_db
def test_make_use_of_custom_credential_type(run_module, organization, admin_user, cred_type):
    result = run_module(
        'credential',
        dict(name='Galaxy Token for Steve', organization=organization.name, credential_type=cred_type.name, inputs={'token': '7rEZK38DJl58A7RxA6EC7lLvUHbBQ1'}),
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', False), result

    cred = Credential.objects.get(name='Galaxy Token for Steve')
    assert cred.credential_type_id == cred_type.id
    assert list(cred.inputs.keys()) == ['token']
    assert cred.inputs['token'].startswith('$encrypted$')
    assert len(cred.inputs['token']) >= len('$encrypted$') + len('7rEZK38DJl58A7RxA6EC7lLvUHbBQ1')

    assert result['name'] == "Galaxy Token for Steve"
    assert result['id'] == cred.pk


@pytest.mark.django_db
@pytest.mark.parametrize('update_secrets', [True, False])
def test_secret_field_write_twice(run_module, organization, admin_user, cred_type, update_secrets):
    val1 = '7rEZK38DJl58A7RxA6EC7lLvUHbBQ1'
    val2 = '7rEZ238DJl5837rxA6xxxlLvUHbBQ1'
    for val in (val1, val2):
        result = run_module(
            'credential',
            dict(
                name='Galaxy Token for Steve',
                organization=organization.name,
                credential_type=cred_type.name,
                inputs={'token': val},
                update_secrets=update_secrets,
            ),
            admin_user,
        )
        assert not result.get('failed', False), result.get('msg', result)

        if update_secrets:
            assert Credential.objects.get(id=result['id']).get_input('token') == val

    if update_secrets:
        assert result.get('changed'), result
    else:
        assert result.get('changed') is False, result
        assert Credential.objects.get(id=result['id']).get_input('token') == val1


@pytest.mark.django_db
@pytest.mark.parametrize('state', ('present', 'absent', 'exists'))
def test_credential_state(run_module, organization, admin_user, cred_type, state):
    result = run_module(
        'credential',
        dict(
            name='Galaxy Token for Steve',
            organization=organization.name,
            credential_type=cred_type.name,
            inputs={'token': '7rEZK38DJl58A7RxA6EC7lLvUHbBQ1'},
            state=state,
        ),
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_controllers_provision_v1
short_description: Resource module for Wireless Controllers Provision V1
description:
- Manage operation create of the resource Wireless Controllers Provision V1.
- This API is used to provision wireless controller.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceId:
    description: DeviceId path parameter. Network Device ID. This value can be obtained
      by using the API call GET /dna/intent/api/v1/network-device/ip-address/${ipAddress}.
    type: str
  interfaces:
    description: Wireless Controllers Provision's interfaces.
    elements: dict
    suboptions:
      interfaceGateway:
        description: Interface Gateway.
        type: str
      interfaceIPAddress:
        description: Interface IP Address.
        type: str
      interfaceName:
        description: Interface Name.
        type: str
      interfaceNetmaskInCIDR:
        description: Interface Netmask In CIDR, range is 1-30.
        type: int
      lagOrPortNumber:
        description: Lag Or Port Number.
        type: int
      vlanId:
        description: VLAN ID range is 1 - 4094.
        type: int
    type: list
  rollingApUpgrade:
    description: Wireless Controllers Provision's rollingApUpgrade.
    suboptions:
      apRebootPercentage:
        description: AP Reboot Percentage. Permissible values - 5, 15, 25.
        type: int
      enableRollingApUpgrade:
        description: True if Rolling AP Upgrade is enabled, else False.
        type: bool
    type: dict
  skipApProvision:
    description: True if Skip AP Provision is enabled, else False.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless WirelessControllerProvisionV1
  description: Complete reference of the WirelessControllerProvisionV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!wireless-controller-provision
notes:
  - SDK Method used are
    wireless.Wireless.wireless_controller_provision_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessControllers/{deviceId}/provision,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_controllers_provision_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceId: string
    interfaces:
    - interfaceGateway: string
      interfaceIPAddress: string
      interfaceName: string
      interfaceNetmaskInCIDR: 0
      lagOrPortNumber: 0
      vlanId: 0
    rollingApUpgrade:
      apRebootPercentage: 0
      enableRollingApUpgrade: true
    skipApProvision: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_inventory_insight_link_mismatch_v1_info
short_description: Information module for Network Device Inventory Insight Link Mismatch V1
description:
- Get all Network Device Inventory Insight Link Mismatch V1.
- Find all devices with link mismatch speed / vlan .
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
    - SiteId path parameter.
    type: str
  offset:
    description:
    - Offset query parameter. Row Number. Default value is 1.
    type: int
  limit:
    description:
    - Limit query parameter. Default value is 500.
    type: int
  category:
    description:
    - Category query parameter. Links mismatch category. Value can be speed-duplex or vlan.
    type: str
  sortBy:
    description:
    - SortBy query parameter. Sort By.
    type: str
  order:
    description:
    - Order query parameter. Order. Value can be asc or desc. Default value is asc.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices InventoryInsightDeviceLinkMismatchAPIV1
  description: Complete reference of the InventoryInsightDeviceLinkMismatchAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!inventory-insight-device-link-mismatch-api
notes:
  - SDK Method used are
    devices.Devices.inventory_insight_device_link_mismatch,

  - Paths used are
    get /dna/intent/api/v1/network-device/insight/{siteId}/device-link,

"""

EXAMPLES = r"""
- name: Get all Network Device Inventory Insight Link Mismatch V1
  cisco.dnac.network_device_inventory_insight_link_mismatch_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    category: string
    sortBy: string
    order: string
    siteId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "endPortAllowedVlanIds": "string",
          "endPortNativeVlanId": "string",
          "startPortAllowedVlanIds": "string",
          "startPortNativeVlanId": "string",
          "linkStatus": "string",
          "endDeviceHostName": "string",
          "endDeviceId": "string",
          "endDeviceIpAddress": "string",
          "endPortAddress": "string",
          "endPortDuplex": "string",
          "endPortId": "string",
          "endPortMask": "string",
          "endPortName": "string",
          "endPortPepId": "string",
          "endPortSpeed": "string",
          "startDeviceHostName": "string",
          "startDeviceId": "string",
          "startDeviceIpAddress": "string",
          "startPortAddress": "string",
          "startPortDuplex": "string",
          "startPortId": "string",
          "startPortMask": "string",
          "startPortName": "string",
          "startPortPepId": "string",
          "startPortSpeed": "string",
          "lastUpdated": "string",
          "numUpdates": 0,
          "avgUpdateFrequency": 0,
          "type": "string",
          "instanceUuid": "string",
          "instanceTenantId": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: configuration_template_deploy_v2
short_description: Resource module for Configuration Template Deploy V2
description:
- Manage operation create of the resource Configuration Template Deploy V2.
- V2 API to deploy a template.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  forcePushTemplate:
    description: ForcePushTemplate flag.
    type: bool
  isComposite:
    description: Composite template flag.
    type: bool
  mainTemplateId:
    description: Main template UUID of versioned template.
    type: str
  memberTemplateDeploymentInfo:
    description: MemberTemplateDeploymentInfo.
    elements: str
    type: list
  targetInfo:
    description: Configuration Template Deploy V2's targetInfo.
    elements: dict
    suboptions:
      hostName:
        description: Hostname of device is required if targetType is MANAGED_DEVICE_HOSTNAME.
        type: str
      id:
        description: UUID of target is required if targetType is MANAGED_DEVICE_UUID.
        type: str
      params:
        description: Template params/values to be provisioned.
        type: dict
      resourceParams:
        description: Resource params to be provisioned. Refer to features page for usage
          details.
        elements: str
        type: list
      type:
        description: Target type of device.
        type: str
      versionedTemplateId:
        description: Versioned templateUUID to be provisioned.
        type: str
    type: list
  templateId:
    description: UUID of template to be provisioned.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates DeployTemplateV2
  description: Complete reference of the DeployTemplateV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!deploy-template-v-2
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.deploy_template_v2,

  - Paths used are
    post /dna/intent/api/v2/template-programmer/template/deploy,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.configuration_template_deploy_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    forcePushTemplate: true
    isComposite: true
    mainTemplateId: string
    memberTemplateDeploymentInfo:
    - string
    targetInfo:
    - hostName: string
      id: string
      params: {}
      resourceParams:
      - string
      type: string
      versionedTemplateId: string
    templateId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



# (c) 2020 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
    name: schedule_rrule
    author: John Westcott IV (@john-westcott-iv)
    short_description: Generate an rrule string which can be used for Schedules
    requirements:
      - pytz
      - python-dateutil >= 2.7.0
    description:
      - Returns a string based on criteria which represents an rrule
    options:
      _terms:
        description:
          - The frequency of the schedule
          - none - Run this schedule once
          - minute - Run this schedule every x minutes
          - hour - Run this schedule every x hours
          - day - Run this schedule every x days
          - week - Run this schedule weekly
          - month - Run this schedule monthly
        required: True
        choices: ['none', 'minute', 'hour', 'day', 'week', 'month']
      start_date:
        description:
          - The date to start the rule
          - Used for all frequencies
          - Format should be YYYY-MM-DD [HH:MM:SS]
        type: str
      timezone:
        description:
          - The timezone to use for this rule
          - Used for all frequencies
          - Format should be as US/Eastern
          - Defaults to America/New_York
        type: str
      every:
        description:
          - The repetition in months, weeks, days hours or minutes
          - Used for all types except none
        type: int
      end_on:
        description:
          - How to end this schedule
          - If this is not defined, this schedule will never end
          - If this is a positive integer, this schedule will end after this number of occurences
          - If this is a date in the format YYYY-MM-DD [HH:MM:SS], this schedule ends after this date
          - Used for all types except none
        type: str
      on_days:
        description:
          - The days to run this schedule on
          - A comma-separated list which can contain values sunday, monday, tuesday, wednesday, thursday, friday
          - Used for week type schedules
      month_day_number:
        description:
          - The day of the month this schedule will run on (0-31)
          - Used for month type schedules
          - Cannot be used with on_the parameter
        type: int
      on_the:
        description:
          - A description on when this schedule will run
          - Two strings separated by a space
          - First string is one of first, second, third, fourth, last
          - Second string is one of sunday, monday, tuesday, wednesday, thursday, friday
          - Used for month type schedules
          - Cannot be used with month_day_number parameters
"""

EXAMPLES = """
    - name: Create a string for a schedule
      debug:
        msg: "{{ query('awx.awx.schedule_rrule', 'none', start_date='1979-09-13 03:45:07') }}"
"""

RETURN = """
_raw:
  description:
    - String in the rrule format
  type: string
"""
import re

from ansible.module_utils.six import raise_from
from ansible.plugins.lookup import LookupBase
from ansible.errors import AnsibleError
from datetime import datetime

try:
    import pytz
    from dateutil import rrule
except ImportError as imp_exc:
    LIBRARY_IMPORT_ERROR = imp_exc
else:
    LIBRARY_IMPORT_ERROR = None


class LookupModule(LookupBase):
    # plugin constructor
    def __init__(self, *args, **kwargs):
        if LIBRARY_IMPORT_ERROR:
            raise_from(AnsibleError('{0}'.format(LIBRARY_IMPORT_ERROR)), LIBRARY_IMPORT_ERROR)
        super().__init__(*args, **kwargs)

        self.frequencies = {
            'none': rrule.DAILY,
            'minute': rrule.MINUTELY,
            'hour': rrule.HOURLY,
            'day': rrule.DAILY,
            'week': rrule.WEEKLY,
            'month': rrule.MONTHLY,
        }

        self.weekdays = {
            'monday': rrule.MO,
            'tuesday': rrule.TU,
            'wednesday': rrule.WE,
            'thursday': rrule.TH,
            'friday': rrule.FR,
            'saturday': rrule.SA,
            'sunday': rrule.SU,
        }

        self.set_positions = {
            'first': 1,
            'second': 2,
            'third': 3,
            'fourth': 4,
            'last': -1,
        }

    @staticmethod
    def parse_date_time(date_string):
        try:
            return datetime.strptime(date_string, '%Y-%m-%d %H:%M:%S')
        except ValueError:
            return datetime.strptime(date_string, '%Y-%m-%d')

    def run(self, terms, variables=None, **kwargs):
        if len(terms) != 1:
            raise AnsibleError('You may only pass one schedule type in at a time')

        frequency = terms[0].lower()

        return self.get_rrule(frequency, kwargs)

    def get_rrule(self, frequency, kwargs):

        if frequency not in self.frequencies:
            raise AnsibleError('Frequency of {0} is invalid'.format(frequency))

        rrule_kwargs = {
            'freq': self.frequencies[frequency],
            'interval': kwargs.get('every', 1),
        }

        # All frequencies can use a start date
        if 'start_date' in kwargs:
            try:
                rrule_kwargs['dtstart'] = LookupModule.parse_date_time(kwargs['start_date'])
            except Exception as e:
                raise_from(AnsibleError('Parameter start_date must be in the format YYYY-MM-DD [HH:MM:SS]'), e)

        # If we are a none frequency we don't need anything else
        if frequency == 'none':
            rrule_kwargs['count'] = 1
        else:
            # All non-none frequencies can have an end_on option
            if 'end_on' in kwargs:
                end_on = kwargs['end_on']
                if re.match(r'^\d+$', end_on):
                    rrule_kwargs['count'] = end_on
                else:
                    try:
                        rrule_kwargs['until'] = LookupModule.parse_date_time(end_on)
                    except Exception as e:
                        raise_from(AnsibleError('Parameter end_on must either be an integer or in the format YYYY-MM-DD [HH:MM:SS]'), e)

            # A week-based frequency can also take the on_days parameter
            if frequency == 'week' and 'on_days' in kwargs:
                days = []
                for day in kwargs['on_days'].split(','):
                    day = day.strip()
                    if day not in self.weekdays:
                        raise AnsibleError('Parameter on_days must only contain values {0}'.format(', '.join(self.weekdays.keys())))
                    days.append(self.weekdays[day])

                rrule_kwargs['byweekday'] = days

            # A month-based frequency can also deal with month_day_number and on_the options
            if frequency == 'month':
                if 'month_day_number' in kwargs and 'on_the' in kwargs:
                    raise AnsibleError('Month based frequencies can have month_day_number or on_the but not both')

                if 'month_day_number' in kwargs:
                    try:
                        my_month_day = int(kwargs['month_day_number'])
                        if my_month_day < 1 or my_month_day > 31:
                            raise Exception()
                    except Exception as e:
                        raise_from(AnsibleError('month_day_number must be between 1 and 31'), e)

                    rrule_kwargs['bymonthday'] = my_month_day

                if 'on_the' in kwargs:
                    try:
                        (occurance, weekday) = kwargs['on_the'].split(' ')
                    except Exception as e:
                        raise_from(AnsibleError('on_the parameter must be two words separated by a space'), e)

                    if weekday not in self.weekdays:
                        raise AnsibleError('Weekday portion of on_the parameter is not valid')
                    if occurance not in self.set_positions:
                        raise AnsibleError('The first string of the on_the parameter is not valid')

                    rrule_kwargs['byweekday'] = self.weekdays[weekday]
                    rrule_kwargs['bysetpos'] = self.set_positions[occurance]

        my_rule = rrule.rrule(**rrule_kwargs)

        # All frequencies can use a timezone but rrule can't support the format that AWX uses.
        # So we will do a string manip here if we need to
        timezone = 'America/New_York'
        if 'timezone' in kwargs:
            if kwargs['timezone'] not in pytz.all_timezones:
                raise AnsibleError('Timezone parameter is not valid')
            timezone = kwargs['timezone']

        # rrule puts a \n in the rule instad of a space and can't handle timezones
        return_rrule = str(my_rule).replace('\n', ' ').replace('DTSTART:', 'DTSTART;TZID={0}:'.format(timezone))
        # AWX requires an interval. rrule will not add interval if it's set to 1
        if kwargs.get('every', 1) == 1:
            return_rrule = "{0};INTERVAL=1".format(return_rrule)

        return return_rrule



#!/usr/bin/python
# -*- coding: utf-8 -*

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: kms_key
version_added: 5.0.0
short_description: Perform various KMS key management tasks
description:
  - Manage role/user access to a KMS key.
  - Not designed for encrypting/decrypting.
  - Prior to release 5.0.0 this module was called M(community.aws.aws_kms).
    The usage did not change.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  alias:
    description:
      - An alias for a key.
      - For safety, even though KMS does not require keys to have an alias, this module expects all
        new keys to be given an alias to make them easier to manage. Existing keys without an alias
        may be referred to by O(key_id). Use M(amazon.aws.kms_key_info) to find key ids.
      - Note that passing a O(key_id) and O(alias) will only cause a new alias to be added, an alias will never be renamed.
      - The V(alias/) prefix is optional.
      - Required if O(key_id) is not given.
    required: false
    aliases:
      - key_alias
    type: str
  key_id:
    description:
      - Key ID or ARN of the key.
      - One of O(alias) or O(key_id) are required.
    required: false
    aliases:
      - key_arn
    type: str
  enable_key_rotation:
    description:
      - Whether the key should be automatically rotated every year.
    required: false
    type: bool
  state:
    description:
      - Whether a key should be present or absent.
      - Note that making an existing key V(absent) only schedules a key for deletion.
      - Passing a key that is scheduled for deletion with O(state=present) will cancel key deletion.
    required: false
    choices:
      - present
      - absent
    default: present
    type: str
  enabled:
    description: Whether or not a key is enabled.
    default: true
    type: bool
  description:
    description:
      - A description of the CMK.
      - Use a description that helps you decide whether the CMK is appropriate for a task.
    type: str
  multi_region:
    description:
      -  Whether to create a multi-Region primary key or not.
    default: false
    type: bool
    version_added: 5.5.0
  pending_window:
    description:
      - The number of days between requesting deletion of the CMK and when it will actually be deleted.
      - Only used when O(state=absent) and the CMK has not yet been deleted.
      - Valid values are between V(7) and V(30) (inclusive).
      - 'See also: U(https://docs.aws.amazon.com/kms/latest/APIReference/API_ScheduleKeyDeletion.html#KMS-ScheduleKeyDeletion-request-PendingWindowInDays)'
    type: int
    aliases: ['deletion_delay']
    version_added: 1.4.0
    version_added_collection: community.aws
  purge_grants:
    description:
      - Whether the O(grants) argument should cause grants not in the list to be removed.
    default: false
    type: bool
  grants:
    description:
      - A list of grants to apply to the key. Each item must contain O(grants.grantee_principal).
        Each item can optionally contain O(grants.retiring_principal), O(grants.operations), O(grants.constraints),
        O(grants.name).
      - O(grants.grantee_principal) and O(grants.retiring_principal) must be ARNs.
      - 'For full documentation of suboptions see the boto3 documentation:'
      - 'U(https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/kms.html#KMS.Client.create_grant)'
    type: list
    elements: dict
    default: []
    suboptions:
        grantee_principal:
            description: The full ARN of the principal being granted permissions.
            required: true
            type: str
        retiring_principal:
            description: The full ARN of the principal permitted to revoke/retire the grant.
            type: str
        operations:
            type: list
            elements: str
            description:
              - A list of operations that the grantee may perform using the CMK.
            choices: ['Decrypt', 'Encrypt', 'GenerateDataKey', 'GenerateDataKeyWithoutPlaintext', 'ReEncryptFrom', 'ReEncryptTo',
                      'CreateGrant', 'RetireGrant', 'DescribeKey', 'Verify', 'Sign']
        constraints:
            description:
              - Constraints is a dict containing V(encryption_context_subset) or V(encryption_context_equals),
                either or both being a dict specifying an encryption context match.
                See U(https://docs.aws.amazon.com/kms/latest/APIReference/API_GrantConstraints.html) or
                U(https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/kms.html#KMS.Client.create_grant)
            type: dict
        name:
            description:
              - A friendly name for the grant.
              - Use this value to prevent the unintended creation of duplicate grants when retrying this request.
            type: str
  policy:
    description:
      - Policy to apply to the KMS key.
      - See U(https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html)
    type: json
  key_spec:
    aliases:
      - customer_master_key_spec
    description:
      - Specifies the type of KMS key to create.
      - The specification is not changeable once the key is created.
    type: str
    default: SYMMETRIC_DEFAULT
    choices: ['SYMMETRIC_DEFAULT', 'RSA_2048', 'RSA_3072', 'RSA_4096', 'ECC_NIST_P256', 'ECC_NIST_P384', 'ECC_NIST_P521', 'ECC_SECG_P256K1']
    version_added: 2.1.0
    version_added_collection: community.aws
  key_usage:
    description:
      - Determines the cryptographic operations for which you can use the KMS key.
      - The usage is not changeable once the key is created.
    type: str
    default: ENCRYPT_DECRYPT
    choices: ['ENCRYPT_DECRYPT', 'SIGN_VERIFY']
    version_added: 2.1.0
    version_added_collection: community.aws
author:
  - Ted Timmons (@tedder)
  - Will Thames (@willthames)
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3

notes:
  - There are known inconsistencies in the amount of time required for updates of KMS keys to be fully reflected on AWS.
    This can cause issues when running duplicate tasks in succession or using the M(amazon.aws.kms_key_info) module to fetch key metadata
    shortly after modifying keys.
    For this reason, it is recommended to use the return data from this module (M(amazon.aws.kms_key)) to fetch a key's metadata.
  - The C(policies) return key was removed in amazon.aws release 8.0.0.
"""

EXAMPLES = r"""
# Create a new KMS key
- amazon.aws.kms_key:
    alias: mykey
    tags:
      Name: myKey
      Purpose: protect_stuff

# Create a new multi-region KMS key
- amazon.aws.kms_key:
    alias: mykey
    multi_region: true
    tags:
      Name: myKey
      Purpose: protect_stuff

# Update previous key with more tags
- amazon.aws.kms_key:
    alias: mykey
    tags:
      Name: myKey
      Purpose: protect_stuff
      Owner: security_team

# Update a known key with grants allowing an instance with the billing-prod IAM profile
# to decrypt data encrypted with the environment: production, application: billing
# encryption context
- amazon.aws.kms_key:
    key_id: abcd1234-abcd-1234-5678-ef1234567890
    grants:
      - name: billing_prod
        grantee_principal: arn:aws:iam::123456789012:role/billing_prod
        constraints:
          encryption_context_equals:
            environment: production
            application: billing
        operations:
          - Decrypt
          - RetireGrant

- name: Update IAM policy on an existing KMS key
  amazon.aws.kms_key:
    alias: my-kms-key
    policy: '{"Version": "2012-10-17", "Id": "my-kms-key-permissions", "Statement": [ { <SOME STATEMENT> } ]}'
    state: present

- name: Example using lookup for policy json
  amazon.aws.kms_key:
    alias: my-kms-key
    policy: "{{ lookup('template', 'kms_iam_policy_template.json.j2') }}"
    state: present
"""

RETURN = r"""
key_id:
  description: ID of key.
  type: str
  returned: always
  sample: "abcd1234-abcd-1234-5678-ef1234567890"
key_arn:
  description: ARN of key.
  type: str
  returned: always
  sample: "arn:aws:kms:ap-southeast-2:123456789012:key/abcd1234-abcd-1234-5678-ef1234567890"
key_state:
  description:
    - The state of the key.
    - Will be one of C('Creating'), C('Enabled'), C('Disabled'), C('PendingDeletion'), C('PendingImport'),
      C('PendingReplicaDeletion'), C('Unavailable'), or C('Updating').
  type: str
  returned: always
  sample: "PendingDeletion"
key_usage:
  description: The cryptographic operations for which you can use the key.
  type: str
  returned: always
  sample: "ENCRYPT_DECRYPT"
origin:
  description: The source of the key's key material. When this value is C(AWS_KMS),
    AWS KMS created the key material. When this value is C(EXTERNAL), the
    key material was imported or the CMK lacks key material.
  type: str
  returned: always
  sample: "AWS_KMS"
aws_account_id:
  description: The AWS Account ID that the key belongs to.
  type: str
  returned: always
  sample: "1234567890123"
creation_date:
  description: Date and time of creation of the key.
  type: str
  returned: always
  sample: "2017-04-18T15:12:08.551000+10:00"
deletion_date:
  description: Date and time after which KMS deletes this KMS key.
  type: str
  returned: when RV(key_state) is PendingDeletion
  sample: "2017-04-18T15:12:08.551000+10:00"
  version_added: 3.3.0
  version_added_collection: community.aws
description:
  description: Description of the key.
  type: str
  returned: always
  sample: "My Key for Protecting important stuff"
enabled:
  description: Whether the key is enabled. True if RV(key_state) is V(Enabled).
  type: bool
  returned: always
  sample: false
enable_key_rotation:
  description: Whether the automatic annual key rotation is enabled. Returns None if key rotation status can't be determined.
  type: bool
  returned: always
  sample: false
aliases:
  description: List of aliases associated with the key.
  type: list
  returned: always
  sample:
    - aws/acm
    - aws/ebs
key_policies:
  description: List of policy documents for the key. Empty when access is denied even if there are policies.
  type: list
  returned: always
  elements: dict
  sample:
    Version: "2012-10-17"
    Id: "auto-ebs-2"
    Statement:
    - Sid: "Allow access through EBS for all principals in the account that are authorized to use EBS"
      Effect: "Allow"
      Principal:
        AWS: "*"
      Action:
      - "kms:Encrypt"
      - "kms:Decrypt"
      - "kms:ReEncrypt*"
      - "kms:GenerateDataKey*"
      - "kms:CreateGrant"
      - "kms:DescribeKey"
      Resource: "*"
      Condition:
        StringEquals:
          kms:CallerAccount: "123456789012"
          kms:ViaService: "ec2.ap-southeast-2.amazonaws.com"
    - Sid: "Allow direct access to key metadata to the account"
      Effect: "Allow"
      Principal:
        AWS: "arn:aws:iam::123456789012:root"
      Action:
      - "kms:Describe*"
      - "kms:Get*"
      - "kms:List*"
      - "kms:RevokeGrant"
      Resource: "*"
  version_added: 3.3.0
  version_added_collection: community.aws
tags:
  description: Dictionary of tags applied to the key. Empty when access is denied even if there are tags.
  type: dict
  returned: always
  sample:
    Name: myKey
    Purpose: protecting_stuff
grants:
  description: List of grants associated with a key.
  type: list
  elements: dict
  returned: always
  contains:
    constraints:
      description: Constraints on the encryption context that the grant allows.
        See U(https://docs.aws.amazon.com/kms/latest/APIReference/API_GrantConstraints.html) for further details
      type: dict
      returned: always
      sample:
        encryption_context_equals:
           "aws:lambda:_function_arn": "arn:aws:lambda:ap-southeast-2:123456789012:function:xyz"
    creation_date:
      description: Date of creation of the grant.
      type: str
      returned: always
      sample: "2017-04-18T15:12:08+10:00"
    grant_id:
      description: The unique ID for the grant.
      type: str
      returned: always
      sample: "abcd1234abcd1234abcd1234abcd1234abcd1234abcd1234abcd1234abcd1234"
    grantee_principal:
      description: The principal that receives the grant's permissions.
      type: str
      returned: always
      sample: "arn:aws:sts::123456789012:assumed-role/lambda_xyz/xyz"
    issuing_account:
      description: The AWS account under which the grant was issued.
      type: str
      returned: always
      sample: "arn:aws:iam::123456789012:root"
    key_id:
      description: The key ARN to which the grant applies.
      type: str
      returned: always
      sample: "arn:aws:kms:ap-southeast-2:123456789012:key/abcd1234-abcd-1234-5678-ef1234567890"
    name:
      description: The friendly name that identifies the grant.
      type: str
      returned: always
      sample: "xyz"
    operations:
      description: The list of operations permitted by the grant.
      type: list
      returned: always
      sample:
        - Decrypt
        - RetireGrant
    retiring_principal:
      description: The principal that can retire the grant.
      type: str
      returned: always
      sample: "arn:aws:sts::123456789012:assumed-role/lambda_xyz/xyz"
changes_needed:
  description: Grant types that would be changed/were changed.
  type: dict
  returned: always
  sample: { "role": "add", "role grant": "add" }
had_invalid_entries:
  description: Whether there are invalid (non-ARN) entries in the KMS entry. These don't count as a change, but will be removed if any changes are being made.
  type: bool
  returned: always
multi_region:
  description:
    - Indicates whether the CMK is a multi-Region C(True) or regional C(False) key.
    - This value is True for multi-Region primary and replica CMKs and False for regional CMKs.
  type: bool
  version_added: 5.5.0
  returned: always
  sample: False
customer_master_key_spec:
  description: Specifies the type of KMS key to create.
  type: str
  returned: always
  sample: "SYMMETRIC_DEFAULT"
encryption_algorithms:
  description: The encryption algorithms that the KMS key supports.
  type: list
  elements: str
  returned: always
  sample: ["SYMMETRIC_DEFAULT"]
key_manager:
  description: The manager of the KMS key.
  type: str
  returned: always
  sample: "AWS"
key_spec:
  description: Specifies the type of KMS key to create.
  type: str
  returned: always
  sample: "SYMMETRIC_DEFAULT"
"""

import json

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def get_kms_keys_with_backoff(connection):
    paginator = connection.get_paginator("list_keys")
    return paginator.paginate().build_full_result()


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def get_kms_aliases_with_backoff(connection):
    paginator = connection.get_paginator("list_aliases")
    return paginator.paginate().build_full_result()


def get_kms_aliases_lookup(connection):
    _aliases = dict()
    for alias in get_kms_aliases_with_backoff(connection)["Aliases"]:
        # Not all aliases are actually associated with a key
        if "TargetKeyId" in alias:
            # strip off leading 'alias/' and add it to key's aliases
            if alias["TargetKeyId"] in _aliases:
                _aliases[alias["TargetKeyId"]].append(alias["AliasName"][6:])
            else:
                _aliases[alias["TargetKeyId"]] = [alias["AliasName"][6:]]
    return _aliases


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def get_kms_tags_with_backoff(connection, key_id, **kwargs):
    return connection.list_resource_tags(KeyId=key_id, **kwargs)


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def get_kms_grants_with_backoff(connection, key_id):
    params = dict(KeyId=key_id)
    paginator = connection.get_paginator("list_grants")
    return paginator.paginate(**params).build_full_result()


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def get_kms_metadata_with_backoff(connection, key_id):
    return connection.describe_key(KeyId=key_id)


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def list_key_policies_with_backoff(connection, key_id):
    paginator = connection.get_paginator("list_key_policies")
    return paginator.paginate(KeyId=key_id).build_full_result()


@AWSRetry.jittered_backoff(retries=5, delay=5, backoff=2.0)
def get_key_policy_with_backoff(connection, key_id, policy_name):
    return connection.get_key_policy(KeyId=key_id, PolicyName=policy_name)


def get_kms_tags(connection, module, key_id):
    # Handle pagination here as list_resource_tags does not have
    # a paginator
    kwargs = {}
    tags = []
    more = True
    while more:
        try:
            tag_response = get_kms_tags_with_backoff(connection, key_id, **kwargs)
            tags.extend(tag_response["Tags"])
        except is_boto3_error_code("AccessDeniedException"):
            tag_response = {}
        except (
            botocore.exceptions.ClientError,
            botocore.exceptions.BotoCoreError,
        ) as e:  # pylint: disable=duplicate-except
            module.fail_json_aws(e, msg="Failed to obtain key tags")
        if tag_response.get("NextMarker"):
            kwargs["Marker"] = tag_response["NextMarker"]
        else:
            more = False
    return tags


def get_kms_policies(connection, module, key_id):
    try:
        policies = list_key_policies_with_backoff(connection, key_id)["PolicyNames"]
        return [get_key_policy_with_backoff(connection, key_id, policy)["Policy"] for policy in policies]
    except is_boto3_error_code("AccessDeniedException"):
        return []
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Failed to obtain key policies")


def camel_to_snake_grant(grant):
    """camel_to_snake_grant snakifies everything except the encryption context"""
    constraints = grant.get("Constraints", {})
    result = camel_dict_to_snake_dict(grant)
    if "EncryptionContextEquals" in constraints:
        result["constraints"]["encryption_context_equals"] = constraints["EncryptionContextEquals"]
    if "EncryptionContextSubset" in constraints:
        result["constraints"]["encryption_context_subset"] = constraints["EncryptionContextSubset"]
    return result


def get_key_details(connection, module, key_id):
    try:
        result = get_kms_metadata_with_backoff(connection, key_id)["KeyMetadata"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to obtain key metadata")
    result["KeyArn"] = result.pop("Arn")

    try:
        aliases = get_kms_aliases_lookup(connection)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to obtain aliases")

    try:
        current_rotation_status = connection.get_key_rotation_status(KeyId=key_id)
        result["enable_key_rotation"] = current_rotation_status.get("KeyRotationEnabled")
    except is_boto3_error_code(["AccessDeniedException", "UnsupportedOperationException"]) as e:
        result["enable_key_rotation"] = None
    result["aliases"] = aliases.get(result["KeyId"], [])

    result = camel_dict_to_snake_dict(result)

    # grants and tags get snakified differently
    try:
        result["grants"] = [
            camel_to_snake_grant(grant) for grant in get_kms_grants_with_backoff(connection, key_id)["Grants"]
        ]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to obtain key grants")
    tags = get_kms_tags(connection, module, key_id)
    result["tags"] = boto3_tag_list_to_ansible_dict(tags, "TagKey", "TagValue")
    policies = get_kms_policies(connection, module, key_id)
    result["key_policies"] = [json.loads(policy) for policy in policies]
    return result


def convert_grant_params(grant, key):
    grant_params = dict(KeyId=key["key_arn"], GranteePrincipal=grant["grantee_principal"])
    if grant.get("operations"):
        grant_params["Operations"] = grant["operations"]
    if grant.get("retiring_principal"):
        grant_params["RetiringPrincipal"] = grant["retiring_principal"]
    if grant.get("name"):
        grant_params["Name"] = grant["name"]
    if grant.get("constraints"):
        grant_params["Constraints"] = dict()
        if grant["constraints"].get("encryption_context_subset"):
            grant_params["Constraints"]["EncryptionContextSubset"] = grant["constraints"]["encryption_context_subset"]
        if grant["constraints"].get("encryption_context_equals"):
            grant_params["Constraints"]["EncryptionContextEquals"] = grant["constraints"]["encryption_context_equals"]
    return grant_params


def different_grant(existing_grant, desired_grant):
    if existing_grant.get("grantee_principal") != desired_grant.get("grantee_principal"):
        return True
    if existing_grant.get("retiring_principal") != desired_grant.get("retiring_principal"):
        return True
    if set(existing_grant.get("operations", [])) != set(desired_grant.get("operations")):
        return True
    if existing_grant.get("constraints") != desired_grant.get("constraints"):
        return True
    return False


def compare_grants(existing_grants, desired_grants, purge_grants=False):
    existing_dict = dict((eg["name"], eg) for eg in existing_grants)
    desired_dict = dict((dg["name"], dg) for dg in desired_grants)
    to_add_keys = set(desired_dict.keys()) - set(existing_dict.keys())
    if purge_grants:
        to_remove_keys = set(existing_dict.keys()) - set(desired_dict.keys())
    else:
        to_remove_keys = set()
    to_change_candidates = set(existing_dict.keys()) & set(desired_dict.keys())
    for candidate in to_change_candidates:
        if different_grant(existing_dict[candidate], desired_dict[candidate]):
            to_add_keys.add(candidate)
            to_remove_keys.add(candidate)

    to_add = []
    to_remove = []
    for key in to_add_keys:
        grant = desired_dict[key]
        to_add.append(grant)
    for key in to_remove_keys:
        grant = existing_dict[key]
        to_remove.append(grant)
    return to_add, to_remove


def start_key_deletion(connection, module, key_metadata):
    if key_metadata["KeyState"] == "PendingDeletion":
        return False

    if module.check_mode:
        return True

    deletion_params = {"KeyId": key_metadata["Arn"]}
    if module.params.get("pending_window"):
        deletion_params["PendingWindowInDays"] = module.params.get("pending_window")

    try:
        connection.schedule_key_deletion(**deletion_params)
        return True
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to schedule key for deletion")


def cancel_key_deletion(connection, module, key):
    key_id = key["key_arn"]
    if key["key_state"] != "PendingDeletion":
        return False

    if module.check_mode:
        return True

    try:
        connection.cancel_key_deletion(KeyId=key_id)
        # key is disabled after deletion cancellation
        # set this so that ensure_enabled_disabled works correctly
        key["key_state"] = "Disabled"
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to cancel key deletion")

    return True


def ensure_enabled_disabled(connection, module, key, enabled):
    desired_state = "Enabled"
    if not enabled:
        desired_state = "Disabled"

    if key["key_state"] == desired_state:
        return False

    key_id = key["key_arn"]
    if not module.check_mode:
        if enabled:
            try:
                connection.enable_key(KeyId=key_id)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to enable key")
        else:
            try:
                connection.disable_key(KeyId=key_id)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to disable key")

    return True


def update_alias(connection, module, key, alias):
    alias = canonicalize_alias_name(alias)

    if alias is None:
        return False

    key_id = key["key_arn"]
    aliases = get_kms_aliases_with_backoff(connection)["Aliases"]
    # We will only add new aliases, not rename existing ones
    if alias in [_alias["AliasName"] for _alias in aliases]:
        return False

    if not module.check_mode:
        try:
            connection.create_alias(TargetKeyId=key_id, AliasName=alias)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed create key alias")

    return True


def update_description(connection, module, key, description):
    if description is None:
        return False
    if key["description"] == description:
        return False

    key_id = key["key_arn"]
    if not module.check_mode:
        try:
            connection.update_key_description(KeyId=key_id, Description=description)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to update key description")

    return True


def update_tags(connection, module, key, desired_tags, purge_tags):
    if desired_tags is None:
        return False

    to_add, to_remove = compare_aws_tags(key["tags"], desired_tags, purge_tags)
    if not (bool(to_add) or bool(to_remove)):
        return False

    key_id = key["key_arn"]
    if not module.check_mode:
        if to_remove:
            try:
                connection.untag_resource(KeyId=key_id, TagKeys=to_remove)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Unable to remove tag")
        if to_add:
            try:
                tags = ansible_dict_to_boto3_tag_list(
                    module.params["tags"],
                    tag_name_key_name="TagKey",
                    tag_value_key_name="TagValue",
                )
                connection.tag_resource(KeyId=key_id, Tags=tags)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Unable to add tag to key")

    return True


def update_policy(connection, module, key, policy):
    if policy is None:
        return False
    try:
        new_policy = json.loads(policy)
    except ValueError as e:
        module.fail_json_aws(e, msg="Unable to parse new policy as JSON")

    key_id = key["key_arn"]
    try:
        keyret = connection.get_key_policy(KeyId=key_id, PolicyName="default")
        original_policy = json.loads(keyret["Policy"])
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError):
        # If we can't fetch the current policy assume we're making a change
        # Could occur if we have PutKeyPolicy without GetKeyPolicy
        original_policy = {}

    if not compare_policies(original_policy, new_policy):
        return False

    if not module.check_mode:
        try:
            connection.put_key_policy(KeyId=key_id, PolicyName="default", Policy=policy)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Unable to update key policy")

    return True


def update_key_rotation(connection, module, key, enable_key_rotation):
    if enable_key_rotation is None:
        return False
    key_id = key["key_arn"]

    try:
        current_rotation_status = connection.get_key_rotation_status(KeyId=key_id)
        if current_rotation_status.get("KeyRotationEnabled") == enable_key_rotation:
            return False
    except is_boto3_error_code("AccessDeniedException"):
        pass
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Unable to get current key rotation status")

    if not module.check_mode:
        try:
            if enable_key_rotation:
                connection.enable_key_rotation(KeyId=key_id)
            else:
                connection.disable_key_rotation(KeyId=key_id)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to enable/disable key rotation")

    return True


def update_grants(connection, module, key, desired_grants, purge_grants):
    existing_grants = key["grants"]

    to_add, to_remove = compare_grants(existing_grants, desired_grants, purge_grants)
    if not (bool(to_add) or bool(to_remove)):
        return False

    key_id = key["key_arn"]
    if not module.check_mode:
        for grant in to_remove:
            try:
                connection.retire_grant(KeyId=key_id, GrantId=grant["grant_id"])
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Unable to retire grant")
        for grant in to_add:
            grant_params = convert_grant_params(grant, key)
            try:
                connection.create_grant(**grant_params)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Unable to create grant")

    return True


def update_key(connection, module, key):
    changed = False

    changed |= cancel_key_deletion(connection, module, key)
    changed |= ensure_enabled_disabled(connection, module, key, module.params["enabled"])
    changed |= update_alias(connection, module, key, module.params["alias"])
    changed |= update_description(connection, module, key, module.params["description"])
    changed |= update_tags(connection, module, key, module.params["tags"], module.params.get("purge_tags"))
    changed |= update_policy(connection, module, key, module.params.get("policy"))
    changed |= update_grants(connection, module, key, module.params.get("grants"), module.params.get("purge_grants"))
    changed |= update_key_rotation(connection, module, key, module.params.get("enable_key_rotation"))

    # make results consistent with kms_facts before returning
    result = get_key_details(connection, module, key["key_arn"])
    result["changed"] = changed
    return result


def create_key(connection, module):
    key_usage = module.params.get("key_usage")
    key_spec = module.params.get("key_spec")
    multi_region = module.params.get("multi_region")
    tags_list = ansible_dict_to_boto3_tag_list(
        module.params["tags"] or {},
        # KMS doesn't use 'Key' and 'Value' as other APIs do.
        tag_name_key_name="TagKey",
        tag_value_key_name="TagValue",
    )
    params = dict(
        BypassPolicyLockoutSafetyCheck=False,
        Tags=tags_list,
        KeyUsage=key_usage,
        CustomerMasterKeySpec=key_spec,
        Origin="AWS_KMS",
        MultiRegion=multi_region,
    )

    if module.check_mode:
        return {"changed": True}

    if module.params.get("description"):
        params["Description"] = module.params["description"]
    if module.params.get("policy"):
        params["Policy"] = module.params["policy"]
    try:
        result = connection.create_key(**params)["KeyMetadata"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to create initial key")

    key = get_key_details(connection, module, result["KeyId"])
    update_alias(connection, module, key, module.params["alias"])
    update_key_rotation(connection, module, key, module.params.get("enable_key_rotation"))

    ensure_enabled_disabled(connection, module, key, module.params.get("enabled"))
    update_grants(connection, module, key, module.params.get("grants"), False)

    # make results consistent with kms_facts
    result = get_key_details(connection, module, key["key_id"])
    result["changed"] = True
    return result


def delete_key(connection, module, key_metadata):
    changed = False

    changed |= start_key_deletion(connection, module, key_metadata)

    result = get_key_details(connection, module, key_metadata["Arn"])
    result["changed"] = changed
    return result


def canonicalize_alias_name(alias):
    if alias is None:
        return None
    if alias.startswith("alias/"):
        return alias
    return "alias/" + alias


def fetch_key_metadata(connection, module, key_id, alias):
    # Note - fetching a key's metadata is very inconsistent shortly after any sort of update to a key has occurred.
    # Combinations of manual waiters, checking expecting key values to actual key value, and static sleeps
    #        have all been exhausted, but none of those available options have solved the problem.
    # Integration tests will wait for 10 seconds to combat this issue.
    # See https://github.com/ansible-collections/community.aws/pull/1052.

    alias = canonicalize_alias_name(module.params.get("alias"))

    try:
        # Fetch by key_id where possible
        if key_id:
            return get_kms_metadata_with_backoff(connection, key_id)["KeyMetadata"]
        # Or try alias as a backup
        return get_kms_metadata_with_backoff(connection, alias)["KeyMetadata"]

    except connection.exceptions.NotFoundException:
        return None
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, "Failed to fetch key metadata.")


def validate_params(module, key_metadata):
    # We can't create keys with a specific ID, if we can't access the key we'll have to fail
    if module.params.get("state") == "present" and module.params.get("key_id") and not key_metadata:
        module.fail_json(msg=f"Could not find key with id {module.params.get('key_id')} to update")
    if module.params.get("multi_region") and key_metadata and module.params.get("state") == "present":
        module.fail_json(msg="You cannot change the multi-region property on an existing key.")


def main():
    argument_spec = dict(
        alias=dict(aliases=["key_alias"]),
        pending_window=dict(aliases=["deletion_delay"], type="int"),
        key_id=dict(aliases=["key_arn"]),
        description=dict(),
        enabled=dict(type="bool", default=True),
        multi_region=dict(type="bool", default=False),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        grants=dict(type="list", default=[], elements="dict"),
        policy=dict(type="json"),
        purge_grants=dict(type="bool", default=False),
        state=dict(default="present", choices=["present", "absent"]),
        enable_key_rotation=(dict(type="bool")),
        key_spec=dict(
            type="str",
            default="SYMMETRIC_DEFAULT",
            aliases=["customer_master_key_spec"],
            choices=[
                "SYMMETRIC_DEFAULT",
                "RSA_2048",
                "RSA_3072",
                "RSA_4096",
                "ECC_NIST_P256",
                "ECC_NIST_P384",
                "ECC_NIST_P521",
                "ECC_SECG_P256K1",
            ],
        ),
        key_usage=dict(
            type="str",
            default="ENCRYPT_DECRYPT",
            choices=["ENCRYPT_DECRYPT", "SIGN_VERIFY"],
        ),
    )

    module = AnsibleAWSModule(
        supports_check_mode=True,
        argument_spec=argument_spec,
        required_one_of=[["alias", "key_id"]],
    )

    kms = module.client("kms")

    key_metadata = fetch_key_metadata(kms, module, module.params.get("key_id"), module.params.get("alias"))
    validate_params(module, key_metadata)

    if module.params.get("state") == "absent":
        if key_metadata is None:
            module.exit_json(changed=False)
        result = delete_key(kms, module, key_metadata)
        module.exit_json(**result)

    if key_metadata:
        key_details = get_key_details(kms, module, key_metadata["Arn"])
        result = update_key(kms, module, key_details)
        module.exit_json(**result)

    result = create_key(kms, module)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# coding: utf-8 -*-


# (c) 2020, John Westcott IV <john.westcott.iv@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: ad_hoc_command
author: "John Westcott IV (@john-westcott-iv)"
version_added: "4.0.0"
short_description: create, update, or destroy Automation Platform Controller ad hoc commands.
description:
    - Create, update, or destroy Automation Platform Controller ad hoc commands. See
      U(https://www.ansible.com/tower) for an overview.
options:
    job_type:
      description:
        - Job_type to use for the ad hoc command.
      type: str
      choices: [ 'run', 'check' ]
    execution_environment:
      description:
        - Execution Environment name, ID, or named URL to use for the ad hoc command.
      required: False
      type: str
    inventory:
      description:
        - Inventory name, ID, or named URL to use for the ad hoc command.
      required: True
      type: str
    limit:
      description:
        - Limit to use for the ad hoc command.
      type: str
    credential:
      description:
        - Credential name, ID, or named URL to use for ad hoc command.
      required: True
      type: str
    module_name:
      description:
        - The Ansible module to execute.
      required: True
      type: str
    module_args:
      description:
        - The arguments to pass to the module.
      type: str
    forks:
      description:
        - The number of forks to use for this ad hoc execution.
      type: int
    verbosity:
      description:
        - Verbosity level for this ad hoc command run
      type: int
      choices: [ 0, 1, 2, 3, 4, 5 ]
    extra_vars:
      description:
        - Extra variables to use for the ad hoc command..
      type: dict
    become_enabled:
      description:
        - If the become flag should be set.
      type: bool
    diff_mode:
      description:
        - Show the changes made by Ansible tasks where supported
      type: bool
    wait:
      description:
        - Wait for the command to complete.
      default: False
      type: bool
    interval:
      description:
        - The interval to request an update from the controller.
      default: 2
      type: float
    timeout:
      description:
        - If waiting for the command to complete this will abort after this
          amount of seconds
      type: int
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Launch an Ad Hoc Command waiting for it to finish
  ad_hoc_command:
    inventory: Demo Inventory
    credential: Demo Credential
    module_name: command
    module_args: echo I <3 Ansible
    wait: true
'''

RETURN = '''
id:
    description: id of the newly launched command
    returned: success
    type: int
    sample: 86
status:
    description: status of newly launched command
    returned: success
    type: str
    sample: pending
'''

from ..module_utils.controller_api import ControllerAPIModule
import json


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        job_type=dict(choices=['run', 'check']),
        inventory=dict(required=True),
        limit=dict(),
        credential=dict(required=True),
        module_name=dict(required=True),
        module_args=dict(),
        forks=dict(type='int'),
        verbosity=dict(type='int', choices=[0, 1, 2, 3, 4, 5]),
        extra_vars=dict(type='dict'),
        become_enabled=dict(type='bool'),
        diff_mode=dict(type='bool'),
        wait=dict(default=False, type='bool'),
        interval=dict(default=2.0, type='float'),
        timeout=dict(type='int'),
        execution_environment=dict(),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    inventory = module.params.get('inventory')
    credential = module.params.get('credential')
    module_name = module.params.get('module_name')
    module_args = module.params.get('module_args')

    wait = module.params.get('wait')
    interval = module.params.get('interval')
    timeout = module.params.get('timeout')
    execution_environment = module.params.get('execution_environment')

    # Create a datastructure to pass into our command launch
    post_data = {
        'module_name': module_name,
        'module_args': module_args,
    }
    for arg in ['job_type', 'limit', 'forks', 'verbosity', 'extra_vars', 'become_enabled', 'diff_mode']:
        if module.params.get(arg):
            # extra_var can receive a dict or a string, if a dict covert it to a string
            if arg == 'extra_vars' and not isinstance(module.params.get(arg), str):
                post_data[arg] = json.dumps(module.params.get(arg))
            else:
                post_data[arg] = module.params.get(arg)

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    post_data['inventory'] = module.resolve_name_to_id('inventories', inventory)
    post_data['credential'] = module.resolve_name_to_id('credentials', credential)
    if execution_environment:
        post_data['execution_environment'] = module.resolve_name_to_id('execution_environments', execution_environment)

    # Launch the ad hoc command
    results = module.post_endpoint('ad_hoc_commands', **{'data': post_data})

    if results['status_code'] != 201:
        module.fail_json(msg="Failed to launch command, see response for details", **{'response': results})

    if not wait:
        module.exit_json(
            **{
                'changed': True,
                'id': results['json']['id'],
                'status': results['json']['status'],
            }
        )

    # Invoke wait function
    results = module.wait_on_url(url=results['json']['url'], object_name=module_name, object_type='Ad Hoc Command', timeout=timeout, interval=interval)

    module.exit_json(
        **{
            'changed': True,
            'id': results['json']['id'],
            'status': results['json']['status'],
        }
    )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The arg spec for the eos_route_maps module
"""


class Route_mapsArgs(object):  # pylint: disable=R0903
    """The arg spec for the eos_route_maps module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "running_config": {"type": "str"},
        "state": {
            "default": "merged",
            "type": "str",
            "choices": [
                "deleted",
                "merged",
                "overridden",
                "replaced",
                "gathered",
                "rendered",
                "parsed",
            ],
        },
        "config": {
            "elements": "dict",
            "type": "list",
            "options": {
                "route_map": {"type": "str"},
                "entries": {
                    "elements": "dict",
                    "type": "list",
                    "options": {
                        "set": {
                            "type": "dict",
                            "options": {
                                "extcommunity": {
                                    "type": "dict",
                                    "options": {
                                        "rt": {
                                            "type": "dict",
                                            "options": {
                                                "vpn": {"type": "str"},
                                                "additive": {"type": "bool"},
                                                "delete": {"type": "bool"},
                                            },
                                        },
                                        "none": {"type": "bool"},
                                        "soo": {
                                            "type": "dict",
                                            "options": {
                                                "vpn": {"type": "str"},
                                                "additive": {"type": "bool"},
                                                "delete": {"type": "bool"},
                                            },
                                        },
                                        "lbw": {
                                            "type": "dict",
                                            "options": {
                                                "aggregate": {"type": "bool"},
                                                "divide": {
                                                    "type": "str",
                                                    "choices": [
                                                        "equal",
                                                        "ration",
                                                    ],
                                                },
                                                "value": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "origin": {
                                    "type": "str",
                                    "choices": ["egp", "igp", "incomplete"],
                                },
                                "isis_level": {"type": "str"},
                                "weight": {"type": "int"},
                                "distance": {"type": "int"},
                                "ip": {
                                    "type": "dict",
                                    "options": {
                                        "peer_address": {"type": "bool"},
                                        "unchanged": {"type": "bool"},
                                        "address": {"type": "str"},
                                    },
                                },
                                "metric": {
                                    "type": "dict",
                                    "options": {
                                        "add": {
                                            "type": "str",
                                            "choices": [
                                                "igp-metric",
                                                "igp-nexthop-cost",
                                            ],
                                        },
                                        "igp_param": {
                                            "type": "str",
                                            "choices": [
                                                "igp-metric",
                                                "igp-nexthop-cost",
                                            ],
                                        },
                                        "value": {"type": "str"},
                                    },
                                },
                                "nexthop": {
                                    "type": "dict",
                                    "options": {
                                        "value": {"type": "int"},
                                        "max_metric": {"type": "bool"},
                                    },
                                },
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "match": {
                                            "type": "dict",
                                            "options": {
                                                "as_number": {"type": "str"},
                                                "none": {"type": "bool"},
                                            },
                                        },
                                        "prepend": {
                                            "type": "dict",
                                            "options": {
                                                "last_as": {"type": "int"},
                                                "as_number": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "community_attributes": {
                                    "type": "dict",
                                    "options": {
                                        "none": {"type": "bool"},
                                        "graceful_shutdown": {"type": "bool"},
                                        "community": {
                                            "type": "dict",
                                            "options": {
                                                "additive": {"type": "bool"},
                                                "local_as": {"type": "bool"},
                                                "no_export": {"type": "bool"},
                                                "list": {"type": "str"},
                                                "number": {"type": "str"},
                                                "no_advertise": {
                                                    "type": "bool",
                                                },
                                                "internet": {"type": "bool"},
                                                "graceful_shutdown": {
                                                    "type": "bool",
                                                },
                                                "delete": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "bgp": {"type": "int"},
                                "tag": {"type": "int"},
                                "local_preference": {"type": "int"},
                                "segment_index": {"type": "int"},
                                "ipv6": {
                                    "type": "dict",
                                    "options": {
                                        "peer_address": {"type": "bool"},
                                        "unchanged": {"type": "bool"},
                                        "address": {"type": "str"},
                                    },
                                },
                                "metric_type": {
                                    "type": "str",
                                    "choices": ["type-1", "type-2"],
                                },
                                "evpn": {"type": "bool"},
                            },
                        },
                        "description": {"type": "str"},
                        "sequence": {"type": "int"},
                        "source": {"type": "dict"},
                        "continue_sequence": {"type": "int"},
                        "statement": {"type": "str"},
                        "action": {
                            "type": "str",
                            "choices": ["deny", "permit"],
                        },
                        "sub_route_map": {
                            "type": "dict",
                            "options": {
                                "name": {"type": "str"},
                                "invert_result": {"type": "bool"},
                            },
                        },
                        "match": {
                            "type": "dict",
                            "options": {
                                "extcommunity": {
                                    "type": "dict",
                                    "options": {
                                        "community_list": {"type": "str"},
                                        "exact_match": {"type": "bool"},
                                    },
                                },
                                "router_id": {"type": "str"},
                                "invert_result": {
                                    "type": "dict",
                                    "options": {
                                        "extcommunity": {
                                            "type": "dict",
                                            "options": {
                                                "community_list": {
                                                    "type": "str",
                                                },
                                                "exact_match": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "large_community": {
                                            "type": "dict",
                                            "options": {
                                                "community_list": {
                                                    "type": "str",
                                                },
                                                "exact_match": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "aggregate_role": {
                                            "type": "dict",
                                            "options": {
                                                "contributor": {
                                                    "type": "bool",
                                                },
                                                "route_map": {"type": "str"},
                                            },
                                        },
                                        "as_path": {
                                            "type": "dict",
                                            "options": {
                                                "path_list": {"type": "str"},
                                                "length": {"type": "str"},
                                            },
                                        },
                                        "community": {
                                            "type": "dict",
                                            "options": {
                                                "community_list": {
                                                    "type": "str",
                                                },
                                                "instances": {"type": "str"},
                                                "exact_match": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                    },
                                },
                                "large_community": {
                                    "type": "dict",
                                    "options": {
                                        "community_list": {"type": "str"},
                                        "exact_match": {"type": "bool"},
                                    },
                                },
                                "ip": {
                                    "type": "dict",
                                    "options": {
                                        "resolved_next_hop": {"type": "str"},
                                        "next_hop": {"type": "str"},
                                        "address": {
                                            "type": "dict",
                                            "options": {
                                                "prefix_list": {"type": "str"},
                                                "dynamic": {"type": "bool"},
                                                "access_list": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "aggregate_role": {
                                    "type": "dict",
                                    "options": {
                                        "contributor": {"type": "bool"},
                                        "route_map": {"type": "str"},
                                    },
                                },
                                "isis_level": {"type": "str"},
                                "community": {
                                    "type": "dict",
                                    "options": {
                                        "community_list": {"type": "str"},
                                        "instances": {"type": "str"},
                                        "exact_match": {"type": "bool"},
                                    },
                                },
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "path_list": {"type": "str"},
                                        "length": {"type": "str"},
                                    },
                                },
                                "route_type": {"type": "str"},
                                "as": {"type": "int"},
                                "tag": {"type": "int"},
                                "local_preference": {"type": "int"},
                                "ipv6": {
                                    "type": "dict",
                                    "options": {
                                        "resolved_next_hop": {"type": "str"},
                                        "next_hop": {"type": "str"},
                                        "address": {
                                            "type": "dict",
                                            "options": {
                                                "prefix_list": {"type": "str"},
                                                "dynamic": {"type": "bool"},
                                                "access_list": {"type": "str"},
                                            },
                                        },
                                    },
                                },
                                "metric_type": {
                                    "type": "str",
                                    "choices": ["type-1", "type-2"],
                                },
                                "interface": {"type": "str"},
                                "source_protocol": {"type": "str"},
                                "metric": {"type": "int"},
                            },
                        },
                    },
                },
            },
        },
    }  # pylint: disable=C0301
